#
#  $Id: test_AD4gpf.py,v 1.13 2012/12/06 23:24:18 annao Exp $
#
#

import unittest, glob, os, sys
from string import strip, find, split, join
import time
from MolKit.molecule import Atom

mv = None
ct = 0
totalCt = 39
klass = None


class ADgpf4_BaseTests(unittest.TestCase):
    """
    setUp + tearDown form a fixture: working environment for the testing code
    """
    
        
    def startViewer(self):
        global mv
        if mv is None:
            from MolKit import Read
            import Tkinter
            from Pmv.moleculeViewer import MoleculeViewer
            mv = MoleculeViewer(trapExceptions=False)
            mv.loadModule('autogpfCommands', 'AutoDockTools')
            mv.loadModule('deleteCommands', 'Pmv')
            mv.loadModule('selectionCommands', 'Pmv')
            #change warningMsg format
            mv.setUserPreference(('warningMsgFormat', 'printed'), log = 0)
        self.mv = mv


    def setUp(self):
        """
        clean-up
        """
        global klass, mv
        name = self.__class__.__name__
        if not klass:
            klass = name
        elif klass != name:
            print 'SETUP: destroying mv'
            if mv and mv.hasGui:
                mv.Exit(0)
                if hasattr(self, 'mv'):
                    del self.mv
                mv = None
            klass = name
            
        if not hasattr(self, 'mv'):
            self.startViewer()
        for m in self.mv.Mols:
            self.mv.deleteMol(m)


    def tearDown(self):
        """
        clean-up
        """
        #print 'in gpf tearDown'
        global ct, totalCt
        #reset gridparameter object
        from AutoDockTools.GridParameters import GridParameters
        self.mv.gpo = GridParameters()
        self.mv.gpo.vf = self.mv
        #delete any molecules left due to errors
        for m in self.mv.Mols:
            self.mv.deleteMol(m)
        self.mv.GPF_FLEXRES_TYPES = []
        self.mv.GPF_RECEPTOR_TYPES = []
        self.mv.GPF_LIGAND_TYPES = []

        ct = ct + 1
        #print 'ct =', ct
        if ct==totalCt:
            print 'destroying mv'
            self.mv.Exit(0)
            del self.mv       
    
        

class ADgpf4_readgpfTests(ADgpf4_BaseTests):
    
    
    def test_gpf4_read_macro_mol(self):
        """readgpfTest1: read a macro mol and check gpo receptor_filename 
        """
        filename = '1ebg_rec.pdbqt'
        outputfilename = 'new_' + filename
        self.mv.ADgpf4_readMacromolecule(filename, filename=outputfilename)
        self.assertEqual(self.mv.gpo.receptor_filename, filename)
    

    def test_gpf4_read_empty_macro_mol(self):
        """readgpfTests2: read a empty macro mol and check that IOError is raised
        """
        self.assertRaises(IOError,self.mv.ADgpf4_readMacromolecule," ")

    
    def test_gpf4_read_non_existent_macro_mol(self):
        """readgpfTests3: read a non-existent macro mol and check that IOError is raised
        """
        self.assertRaises(IOError, self.mv.ADgpf4_readMacromolecule,"gfdjs.pdbqt")

       
    def test_gpf4_choose_macro_mol_name(self):
        """readgpfTests4: readgpfTests: check chosen macrol name==gpo.receptor_stem
        """
        self.mv.readMolecule('1ebg_rec.pdbqt')
        macro_name = self.mv.Mols[0].name
        self.mv.ADgpf4_chooseMacromolecule(macro_name)
        self.assertEqual(self.mv.gpo.receptor_stem, macro_name)

    
    def test_gpf4_set_map_types_directly(self):
        """readgpfTests5: check for setting map types directly
        """
        teststring = "A B C D E"
        self.mv.ADgpf4_setMapTypes(teststring)
        self.assertEqual(self.mv.gpo['ligand_types']['value'], teststring)
                
 
    def test_gpf4_choose_ligand_name(self):
        """readgpfTests6: choose ligand then check ligand name
        """
        filename = "1ebg_lig.pdbqt"
        self.mv.readMolecule(filename)
        ligand_name = self.mv.Mols[0].name
        self.mv.ADgpf4_chooseFormattedLigand(ligand_name)
        self.assertEqual(self.mv.gpo.ligand.name, ligand_name)


    def test_gpf4_read_formatted_ligand(self):
        """readgpfTests7: read ligand then check ligand name
        """
        filename = "1ebg_lig.pdbqt"
        self.mv.ADgpf4_readFormattedLigand(filename)
        self.assertEqual(self.mv.gpo.ligand_filename, filename)

    
    def test_gpf4_set_map_types_from_ligand(self):
        """readgpfTests8: check for setting map types from ligandfile
        """
        filename = "1ebg_lig.pdbqt"
        self.mv.ADgpf4_readFormattedLigand(filename)
        stored_values = "P C OA HD N"
        #stored_values = "C HD N OA P"
        self.assertEqual(self.mv.gpo['ligand_types']['value'], stored_values)
                

    def test_gpf4_read_empty_formatted_ligand(self):
        """readgpfTests9: check empty ligand name raises IOError
        """
        self.assertRaises(IOError,self.mv.ADgpf4_readFormattedLigand," ")
        
    
    def test_gpf4_read_non_existent_formatted_ligand(self):
        """readgpfTests10: check non_existent ligand name raises IOError
        """
        self.assertRaises(IOError,self.mv.ADgpf4_readFormattedLigand,"hjgf.pdbq ")
       
         
    
#    def test_gpf4_set_up_covalent_map_coords(self):
#        """ check covalent coords  in set up covalent map
#        """
#        self.mv.ADgpf4_readMacromolecule('1ebg_rec.pdbqt')
#        self.mv.ADgpf4_readFormattedLigand('ind.out.pdbq')
#        #c={}
#        #c['covalent_coords']='-3.884'
#        #apply(self.mv.ADgpf4_setUpCovalentMap,(),c)
#        self.mv.ADgpf4_setUpCovalentMap('-22.9870 -0.0600 4.4430',-1.0,'5.0','1000.0','CAONSHZ')
#        self.assertEqual(str(self.mv.gpo['covalent_coords']['value']),'-22.9870 -0.0600 4.4430')

    
    
    def test_gpf4_write_gpf(self):
        """readgpfTests11: check using ref_1ebg.gpf by comparing ref_1ebg.gpf + test_1ebg_rec.gpf
        """
        self.mv.ADgpf4_readMacromolecule('1ebg_rec.pdbqt')
        self.mv.ADgpf4_readFormattedLigand('1ebg_lig.pdbqt')
        self.mv.ADgpf4_writeGPF("test_1ebg_rec.gpf")
        testptr = open("test_1ebg_rec.gpf")
        testlines = testptr.readlines()
        refptr = open("ref_1ebg.gpf")
        reflines = refptr.readlines()
        for refline,testline in zip(reflines,testlines):
            self.assertEqual(refline, testline)


    def test_gpf4_init_Macro_name(self):
        """readgpfTests12: check macro mol name read by init macro
        """
        self.mv.ADgpf4_readMacromolecule("1ebg_rec.pdbqt")
        self.assertEqual(self.mv.gpo.receptor_stem,'1ebg_rec')
        

    def test_gpf4_check_macro_types(self):
        """readgpfTests13: check macro types
        """
        self.mv.ADgpf4_readMacromolecule("1ebg_rec.pdbqt")
        mol = self.mv.Mols[0]
        d = {}
        #build a list of atom autodock types for the mol
        for a in mol.allAtoms:
            d[a.autodock_element] = 1
        types = d.keys() #list of types in macro
        #checking that the gpo has all of those values
        for ch in split(self.mv.gpo['receptor_types']['value']):
            self.assertEqual(ch in types, True)


class ADgpf4_flexResTests(ADgpf4_BaseTests):

    #tests based on Floren's problem
    def test_gpf4_lig_flexres(self):
        """flexResTests: read a ligand file then a flexres file and check written types
        """
        rigid_filename = 'CathepsinB_rigid.pdbqt'
        flexres_filename = 'CathepsinB_flex.pdbqt'
        ligand_filename = 'FGA3.pdbqt'
        output_filename = 'lig_flexres.gpf'
        self.mv.ADgpf4_readMacromolecule(rigid_filename)
        self.mv.ADgpf4_readFormattedLigand(ligand_filename)
        self.mv.ADgpf4_readFormattedFlexRes(flexres_filename)
        self.mv.ADgpf4_writeGPF(output_filename)
        fptr = open(output_filename)
        lines = fptr.readlines()
        types_line = lines[4]
        ref_line ="ligand_types C SA A OA HD N          # ligand atom types\n"
        self.assertEqual(types_line,ref_line)


    def test_gpf4_flexres_lig(self):
        """flexResTests: read a flexres file then a ligand file and check written types
        """
        rigid_filename = 'CathepsinB_rigid.pdbqt'
        flexres_filename = 'CathepsinB_flex.pdbqt'
        ligand_filename = 'FGA3.pdbqt'
        output_filename = 'lig_flexres.gpf'
        self.mv.ADgpf4_readMacromolecule(rigid_filename)
        self.mv.ADgpf4_readFormattedFlexRes(flexres_filename)
        self.mv.ADgpf4_readFormattedLigand(ligand_filename)
        self.mv.ADgpf4_writeGPF(output_filename)
        fptr = open(output_filename)
        lines = fptr.readlines()
        types_line = lines[4]
        ref_line ="ligand_types A C OA HD N SA          # ligand atom types\n"
        self.assertEqual(types_line,ref_line)


    #read receptor
    def test_gpf4_read_rigid_macro_mol(self):
        """flexResTests1: read a rigid receptor and check gpo receptor_filename 
        """
        filename = 'hsg1_rigid.pdbqt'
        self.mv.ADgpf4_readMacromolecule(filename)
        self.assertEqual(self.mv.gpo.receptor_filename,filename)
        args = self.mv.Mols[0].chains.residues.get(lambda x:x.name=='ARG8')
        #check that 5 atoms only are built for the ARG8 residues
        self.assertEqual(len(args.atoms), 10)


    #read ligand
    def test_gpf4_read_flex_mol(self):
        """flexResTests2: read flexible input file and check gpo ligand_filename 
        """
        filename = 'hsg1_flex.pdbqt'
        self.mv.ADgpf4_readFormattedLigand(filename)
        ligand_name = self.mv.Mols[0].name
        self.assertEqual(self.mv.gpo.ligand.name, ligand_name)
        self.assertEqual('ARG8' in self.mv.Mols[0].chains.residues.name, True)
        args = self.mv.Mols[0].chains.residues.get(lambda x:x.name=='ARG8')
        self.assertEqual(len(args), 2)
        self.assertEqual(len(args.atoms.bonds[0])>0, True)
    

    #choose ligand
    def test_gpf4_choose_flex_mol(self):
        """flexResTests3: read in then choose flexible input file. 
        Check ligand_filename in mv.gpo object
        """
        filename = 'hsg1_flex.pdbqt'
        self.mv.readMolecule(filename)
        ligand_name = self.mv.Mols[0].name
        self.assertEqual('ARG8' in self.mv.Mols[0].chains.residues.name, True)
        args = self.mv.Mols[0].chains.residues.get(lambda x:x.name=='ARG8')
        self.assertEqual(len(args), 2)
        self.assertEqual(len(args.atoms.bonds[0])>0, True)
        self.mv.ADgpf4_chooseFormattedLigand(ligand_name)
        self.assertEqual(self.mv.gpo.ligand.name, ligand_name)


    #choose receptor
    def test_gpf4_choose_rigid_mol(self):
        """flexResTests4: readin then choose rigid input file. check gpo macro_filename 
        """
        filename = 'hsg1_rigid.pdbqt'
        self.mv.readMolecule(filename)
        macro_name = self.mv.Mols[0].name
        self.assertEqual('ARG8' in self.mv.Mols[0].chains.residues.name, True)
        args = self.mv.Mols[0].chains.residues.get(lambda x:x.name=='ARG8')
        self.assertEqual(len(args), 2)
        #check that 5 atoms only are built for the ARG8 residues
        self.assertEqual(len(args.atoms), 10)
        self.assertEqual(len(args.atoms.bonds[0])>0, True)
        self.mv.ADgpf4_chooseMacromolecule(macro_name)
        self.assertEqual(self.mv.gpo.receptor_stem, macro_name)


class ADgpf4_setting_flex_types_tests(ADgpf4_BaseTests):

    def setUp(self):
        """
        clean-up
        """
        global klass, mv
        name = self.__class__.__name__
        if not klass:
            klass = name
        elif klass != name:
            print 'SETUP: destroying mv'
            if mv and mv.hasGui:
                mv.Exit(0)
                if hasattr(self, 'mv'):
                    del self.mv
                mv = None
            klass = name
            from AutoDockTools import autoflexCommands
            reload(autoflexCommands)
        if not hasattr(self, 'mv'):
            self.startViewer()
            
        for m in self.mv.Mols:
            self.mv.deleteMol(m)
        self.mv.loadModule('autoflexCommands', 'AutoDockTools')
        self.mv.ADgpf4_readMacromolecule('hsg1.pdbqt')
        #self.mv.ADflex_setResidues("::ARG8")
        #self.mv.ADflex_writeFlexFile('temp_FLEX.pdbqt')
        #self.mv.ADflex_writeRigidFile('temp_RIGID.pdbqt')

    
    def test_gpf4_set_map_types_directly(self):
        """typesTest0: check for setting map types directly
        """
        teststring = "A B C D E"
        self.mv.ADgpf4_setMapTypes(teststring)
        testlist = teststring.split()
        self.assertEqual(self.mv.GPF_LIGAND_TYPES, testlist)
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)
                
 
    def test_gpf4_choose_ligand_name(self):
        """typesTests1: choose ligand then check GPF_LIGAND_TYPES for
        autodock_element of each atom 
        """
        filename = "1ebg_lig.pdbqt"
        self.mv.readMolecule(filename)
        ligand_name = self.mv.Mols[0].name
        self.mv.ADgpf4_chooseFormattedLigand(ligand_name)
        d = {}
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        for k in d.keys():    
            self.assertEqual(k in self.mv.GPF_LIGAND_TYPES, True)
        teststring = join(self.mv.GPF_LIGAND_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)


    def test_gpf4_read_formatted_ligand(self):
        """typesTest2: read ligand then check ligand name
        """
        filename = "1ebg_lig.pdbqt"
        self.mv.ADgpf4_readFormattedLigand(filename)
        d = {}
        for a in self.mv.Mols[-1].allAtoms:
            d[a.autodock_element] = 1
        for k in d.keys():    
            print 'testing k=', k
            self.assertEqual(k in self.mv.GPF_LIGAND_TYPES, True)
        teststring = join(self.mv.GPF_LIGAND_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)


    def test_gpf4_set_map_types_from_flex_setup_residues(self):
        """typesTest3: check for setting map types from newly 
                       formatted flexible residues
        """
        filename = 'hsg1.pdbqt'
        self.mv.readMolecule(filename)
        macro_name = "hsg1"
        self.mv.ADgpf4_chooseMacromolecule(macro_name)
        self.mv.ADflex_setResidues("::MET46")
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:MET46:CA,CB;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:MET46:CA,CB;", False)
        atom_types =['C','SA']
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)


    def test_gpf4_set_map_types_from_flex_file(self):
        """typesTest4: readFormattedFlexRes
        check for setting map types from flexible residues file
        """
        self.mv.ADgpf4_readFormattedFlexRes("met46_flex.pdbqt")
        atom_types =['C','SA']
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(self.mv.GPF_FLEXRES_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)


    def test_gpf4_set_map_types_by_choosing_flex(self):
        """typesTest5: _chooseFormattedFlexRes
        check for setting map types by choosing flexible residues 'mol'
        """
        filename = 'met46_flex.pdbqt'
        self.mv.readMolecule(filename)
        self.mv.ADgpf4_chooseFormattedFlexRes("met46_flex")
        atom_types =['C','SA']
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(atom_types, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)

 
    def test_gpf4_choose_ligand_then_currently_formatted_flex(self):
        """typesTest6: choose ligand then set flex res from currently
        formatted molecule and check GPF_LIGAND_TYPES for
        autodock_element of each atom 
        """
        filename = "1ebg_lig.pdbqt"
        self.mv.readMolecule(filename)
        ligand_name = self.mv.Mols[0].name
        self.mv.ADgpf4_chooseFormattedLigand(ligand_name)
        d = {}
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        filename = 'hsg1.pdbqt'
        self.mv.readMolecule(filename)
        macro_name = "hsg1"
        self.mv.ADgpf4_chooseMacromolecule(macro_name)
        self.mv.ADflex_setResidues("::MET46")
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:MET46:CA,CB;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:MET46:CA,CB;", False)
        d['C'] = 1
        d['SA'] = 1
        atom_types = d.keys()
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(self.mv.GPF_LIGAND_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)


    def test_gpf4_choose_ligand_then_read_formatted_flex(self):
        """typesTest7: choose ligand then set flex res from file
        and check GPF_LIGAND_TYPES for autodock_element of each atom 
        """
        filename = "1ebg_lig.pdbqt"
        self.mv.readMolecule(filename)
        ligand_name = self.mv.Mols[0].name
        self.mv.ADgpf4_chooseFormattedLigand(ligand_name)
        d = {}
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        self.mv.ADgpf4_readFormattedFlexRes('met46_flex.pdbqt')
        d['C'] = 1
        d['SA'] = 1
        atom_types = d.keys()
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        gpo_types  = self.mv.gpo['ligand_types']['value'].split()
        for x in atom_types:
            self.assertEqual(x in gpo_types, True)
        for t in gpo_types:
            self.assertEqual(t in atom_types, True)


    def test_gpf4_choose_ligand_then_choose_formatted_flex(self):
        """typesTest8: choose ligand then choose flex res in viewer
        and check GPF_LIGAND_TYPES for autodock_element of each atom 
        """
        filename = "1ebg_lig.pdbqt"
        self.mv.readMolecule(filename)
        ligand_name = self.mv.Mols[0].name
        self.mv.ADgpf4_chooseFormattedLigand(ligand_name)
        d = {}
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        self.mv.readMolecule("met46_flex.pdbqt")
        self.mv.ADgpf4_chooseFormattedFlexRes('met46_flex.pdbqt')
        d['C'] = 1
        d['SA'] = 1
        atom_types = d.keys()
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(self.mv.GPF_LIGAND_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)

 
    def test_gpf4_read_ligand_then_currently_formatted_flex(self):
        """typesTest9: read ligand then set flex res from currently
        formatted molecule and check GPF_LIGAND_TYPES for
        autodock_element of each atom 
        """
        filename = "1ebg_lig.pdbqt"
        self.mv.ADgpf4_readFormattedLigand(filename)
        d = {}
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        filename = 'hsg1.pdbqt'
        self.mv.readMolecule(filename)
        macro_name = "hsg1"
        self.mv.ADgpf4_chooseMacromolecule(macro_name)
        self.mv.ADflex_setResidues("::MET46")
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:MET46:CA,CB;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:MET46:CA,CB;", False)
        d['C'] = 1
        d['SA'] = 1
        atom_types = d.keys()
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(self.mv.GPF_LIGAND_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)


    def test_gpf4_read_ligand_then_read_formatted_flex(self):
        """typesTest10: read ligand then read flex res file
        and check GPF_LIGAND_TYPES for autodock_element of each atom 
        """
        filename = "1ebg_lig.pdbqt"
        self.mv.ADgpf4_readFormattedLigand(filename)
        d = {}
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        self.mv.ADgpf4_readFormattedFlexRes('met46_flex.pdbqt')
        d['C'] = 1
        d['SA'] = 1
        atom_types = d.keys()
        #msg = 'atom_types =' + str(atom_types)
        #self.mv.warningMsg(msg)
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(atom_types, ' ')
        for x in atom_types:
            gpo_val = self.mv.gpo['ligand_types']['value']
            gpo_types = gpo_val.split()
            #msg = 'testing x ='+ x+ ' is in '+ gpo_val
            #self.mv.warningMsg( msg)
            self.assertEqual(x in atom_types, True)



    def test_gpf4_read_ligand_then_choose_formatted_flex(self):
        """typesTest11: read ligand then choose flex res molecule
        and check GPF_LIGAND_TYPES for autodock_element of each atom 
        """
        filename = "1ebg_lig.pdbqt"
        self.mv.ADgpf4_readFormattedLigand(filename)
        d = {}
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        self.mv.readMolecule("met46_flex.pdbqt")
        self.mv.ADgpf4_chooseFormattedFlexRes('met46_flex.pdbqt')
        d['C'] = 1
        d['SA'] = 1
        atom_types = d.keys()
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(self.mv.GPF_LIGAND_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)

 
 #------------------------------------------------------------------

    def test_gpf4_currently_formatted_flex_then_choose_ligand(self):
        """typesTest12: set flex res from currently formatted molecule 
        then choose ligand and check GPF_LIGAND_TYPES for
        autodock_element of each atom 
        """
        filename = 'hsg1.pdbqt'
        self.mv.readMolecule(filename)
        macro_name = "hsg1"
        self.mv.ADgpf4_chooseMacromolecule(macro_name)
        self.mv.ADflex_setResidues("::MET46")
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:MET46:CA,CB;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:MET46:CA,CB;", False)
        d = {}
        d['C'] = 1
        d['SA'] = 1
        filename = "1ebg_lig.pdbqt"
        self.mv.readMolecule(filename)
        ligand_name = self.mv.Mols[0].name
        self.mv.ADgpf4_chooseFormattedLigand(ligand_name)
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        atom_types = d.keys()
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(self.mv.GPF_LIGAND_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)


    def test_gpf4_read_formatted_flex_then_choose_ligand(self):
        """typesTest13: read flex res from file then choose ligand 
        and check GPF_LIGAND_TYPES for autodock_element of each atom 
        """
        self.mv.ADgpf4_readFormattedFlexRes('met46_flex.pdbqt')
        d = {}
        d['C'] = 1
        d['SA'] = 1
        filename = "1ebg_lig.pdbqt"
        self.mv.readMolecule(filename)
        ligand_name = self.mv.Mols[0].name
        self.mv.ADgpf4_chooseFormattedLigand(ligand_name)
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        atom_types = d.keys()
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(self.mv.GPF_LIGAND_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)


    def test_gpf4_choose_formatted_flex_then_choose_ligand(self):
        """typesTest14: choose flex res in viewer then choose ligand 
        and check GPF_LIGAND_TYPES for autodock_element of each atom 
        """
        self.mv.readMolecule("met46_flex.pdbqt")
        self.mv.ADgpf4_chooseFormattedFlexRes('met46_flex.pdbqt')
        d = {}
        d['C'] = 1
        d['SA'] = 1
        filename = "1ebg_lig.pdbqt"
        self.mv.readMolecule(filename)
        ligand_name = self.mv.Mols[0].name
        self.mv.ADgpf4_chooseFormattedLigand(ligand_name)
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        atom_types = d.keys()
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(self.mv.GPF_LIGAND_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)

 
    def test_gpf4_currently_formatted_flex_then_read_ligand(self):
        """typesTest15: set flex res from currently formatted 
        molecule then read ligand and check GPF_LIGAND_TYPES for
        autodock_element of each atom 
        """
        filename = 'hsg1.pdbqt'
        self.mv.readMolecule(filename)
        macro_name = "hsg1"
        self.mv.ADgpf4_chooseMacromolecule(macro_name)
        self.mv.ADflex_setResidues("::MET46")
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:MET46:CA,CB;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:MET46:CA,CB;", False)
        d = {}
        d['C'] = 1
        d['SA'] = 1
        filename = "1ebg_lig.pdbqt"
        self.mv.ADgpf4_readFormattedLigand(filename)
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        atom_types = d.keys()
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(self.mv.GPF_LIGAND_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)


    def test_gpf4_read_formatted_flex_then_read_ligand(self):
        """typesTest16: read flex res file then read ligand 
        and check GPF_LIGAND_TYPES for autodock_element of each atom 
        """
        self.mv.ADgpf4_readFormattedFlexRes('met46_flex.pdbqt')
        d = {}
        d['C'] = 1
        d['SA'] = 1
        filename = "1ebg_lig.pdbqt"
        self.mv.ADgpf4_readFormattedLigand(filename)
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        atom_types = d.keys()
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(self.mv.GPF_LIGAND_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)


    def test_gpf4_choose_formatted_flex_then_read_ligand(self):
        """typesTest17: choose flex res molecule then read ligand 
        and check GPF_LIGAND_TYPES for autodock_element of each atom 
        """
        self.mv.readMolecule("met46_flex.pdbqt")
        self.mv.ADgpf4_chooseFormattedFlexRes('met46_flex.pdbqt')
        d = {}
        d['C'] = 1
        d['SA'] = 1
        filename = "1ebg_lig.pdbqt"
        self.mv.ADgpf4_readFormattedLigand(filename)
        for a in self.mv.Mols[0].allAtoms:
            d[a.autodock_element] = 1
        atom_types = d.keys()
        for x in self.mv.GPF_FLEXRES_TYPES:
            self.assertEqual(x in atom_types, True)
        teststring = join(self.mv.GPF_LIGAND_TYPES, ' ')
        self.assertEqual(self.mv.gpo['ligand_types']['value'],teststring)



class ADgpf4_formatting_flexResTests(ADgpf4_BaseTests):

    def setUp(self):
        """
        clean-up
        """
        global klass, mv
        name = self.__class__.__name__
        if not klass:
            klass = name
        elif klass != name:
            print 'SETUP: destroying mv'
            if mv and mv.hasGui:
                mv.Exit(0)
                if hasattr(self, 'mv'):
                    del self.mv
                mv = None
            klass = name
            
        if not hasattr(self, 'mv'):
            self.startViewer()
        for m in self.mv.Mols:
            self.mv.deleteMol(m)
        self.mv.loadModule('autoflexCommands', 'AutoDockTools')
        self.mv.readMolecule('hsg1.pdbqt', ask=0, parser=None)
        self.mv.ADflex_chooseMacro("hsg1")
        from MolKit.molecule import Atom
        self.mv.setIcomLevel(Atom, KlassSet=None)
        self.mv.ADflex_setResidues("::ARG8")
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:ARG8:NE,CZ;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:ARG8:CZ,NH1;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:ARG8:CZ,NH2;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:ARG8:NE,CZ;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:ARG8:CZ,NH1;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:ARG8:CZ,NH2;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:B:ARG8:CA,CB;", False)
        self.mv.ADflex_setBondRotatableFlag("hsg1:A:ARG8:CA,CB;", False)
        self.mv.ADflex_writeFlexFile('temp_FLEX.pdbqt')
        self.mv.ADflex_writeRigidFile('temp_RIGID.pdbqt')


    #choose ligand
    #def test_gpf4_choose_flex_mol(self):
    #    """choose new flexible file. check gpo ligand_filename 
    #    """
    #    ligand_name = 'ind'
    #    #oldval = self.mv.gpo['ligand_types']['value']
    #    written_ligand_file = "temp_FLEX.pdbqt"
    #    self.mv.ADgpf4_chooseFormattedLigand(ligand_name)
    #    self.assertEqual(self.mv.gpo.ligand_filename, written_ligand_file)


    #choose receptor
    def test_gpf4_choose_rigid_mol(self):
        """choose new rigid file. check gpo macro_filename 
        """
        macro_name = "hsg1"
        written_macro_name = "temp_RIGID"
        self.mv.ADgpf4_chooseMacromolecule(macro_name)
        self.assertEqual(self.mv.gpo.receptor_stem, written_macro_name)
        self.assertEqual(self.mv.gpo['gridfld']['value'], written_macro_name+".maps.fld")
        self.assertEqual(self.mv.gpo['dsolvmap']['value'], written_macro_name+".d.map")
        self.assertEqual(self.mv.gpo['elecmap']['value'], written_macro_name+".e.map")


class ADgpf4_write_Covalent_GPF_Tests(ADgpf4_BaseTests):
    def setUp(self):
        ADgpf4_BaseTests.setUp(self)
        from AutoDockTools import autogpfCommands
        reload(autogpfCommands)

    def test_gpf4_write_covalent_gpf(self):
        """covalent_map: (1)read rec and lig files,(2)setup covalent atom
        (3) setup map location, (4)write gpf (5)FINALLY compare with ref gpf
        """
        self.mv.ADgpf4_readMacromolecule('cov_rec.pdbqt', preserve_input_charges=1, log=0)
        self.mv.ADgpf4_readFormattedLigand('cov_lig.pdbqt', log=0)
        self.mv.ADgpf4_readFormattedFlexRes('cov_flex.pdbqt', log=0)
        #self.mv.select("LR_cov:d:****:C;",log=0)
        self.mv.ADgpf_setGpo( covalentmap=1, covalent_coords='20.0000 19.0000 18.0000', 
                ligand_types='C SA HD OA N A Z', covalent_half_width='5.0', 
                covalent_energy_barrier='1000.0')
        self.mv.ADgpf_selectCenter("cov_flex:A:PHE154:CD2", log=0)
        self.mv.ADgpf_setGpo(gridcenterAuto=0, gridcenter=[16.284, 73.749, 28.198], log=0, npts=[60, 60, 60])
        self.mv.ADgpf4_writeGPF('test_cov.gpf', log=0)
        #then compare
        fptr = open("ref_cov.gpf")
        reflines = fptr.readlines()
        fptr.close()
        fptr = open("test_cov.gpf")
        testlines = fptr.readlines()
        fptr.close()
        for rline,tline in zip(reflines, testlines):
            self.assertEqual(rline,tline)


if __name__ == '__main__':
    #unittest.main()
    test_cases = [
        'ADgpf4_BaseTests',
        'ADgpf4_readgpfTests',
        'ADgpf4_flexResTests',
        'ADgpf4_setting_flex_types_tests',
        'ADgpf4_formatting_flexResTests',
        'ADgpf4_write_Covalent_GPF_Tests',
            ]
    unittest.main( argv=([__name__ ,] + test_cases) )






#############################################################################
#
# Author: J. Ren
#
#############################################################################

"""
Ligand Library Object
"""

import os
import tempfile
import zipfile
import string, types, re
from AutoDockTools.filterLigands import CalculateProperties, PropertyTable, LigandList, FilterLigands

class LigandDB:
    """Ligand Library Object
    """
    
#    def __init__(self, server_lib=None, url_lib=None, filter_file=None, accepted=None):
    def __init__(self, server_lib=None, url_lib=None):

        self.server_lib = server_lib
        self.url_lib = url_lib
#        self.accepted = accepted
        self.accepted = None
        self.property_file = None
        self.propertyTable = None

#        if filter_file == None:
#            self.filter_file = os.path.abspath('filtered_ligands.txt')
#        else:
#            self.filter_file = os.path.abspath(filter_file)
        self.filter_file = None

        if self.server_lib != None:
            import urllib
            slu = "http://kryptonite.nbcr.net/ligand_props/" + server_lib + ".prop"
            pt = PropertyTable(url=slu)
            self.propertyTable = pt
            self.loc = self.server_lib
            self.loc_type = "server_lib"
        elif self.url_lib != None:
            import urllib
            slu = url_lib + "/lib.prop"
            #print slu
            pt = PropertyTable(url=slu)
            self.propertyTable = pt  
            self.loc = self.url_lib
            self.loc_type = "url_lib"
        
#        if accepted == None:
#            lfilter = FilterLigands()
#            accepted, rejected = lfilter.filterTable(self.propertyTable) 
#            self.SetAcceptedLigands(accepted.filenames)
#        else:
#            f = open(self.filter_file, 'w')
#            for i in self.accepted:
#                f.write(i + '''
#''')
#            f.close()

    def SetAcceptedLigands(self, accept_list, filter_file=None):
        if filter_file == None:
            workingDir = tempfile.mkdtemp()
            filter_file = workingDir + os.sep + 'filtered_ligands.txt'

        self.accepted = accept_list
        self.filter_file = filter_file

        f = open(self.filter_file, 'w')
        for i in self.accepted:
                f.write(i + '''
''')
        f.close()

    def GetPropertyFile(self):
        return self.property_file





#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/compute_AD42ScoreC.py,v 1.3 2013/07/09 17:02:31 rhuey Exp $
#

import os 

from MolKit import Read
import MolKit.molecule
import MolKit.protein
#?@@?
from AutoDockFR import evaluateEnergy
from AutoDockFR.ADCscorer import AD42ScoreC
#rigidRecAtoms, ligAtoms, ligandTorTree, TORSDOF, flexRecAtoms=None,
#		cutoff=1.0, RR_L=True, FR_L=True, L_L=True, RR_RR=True, RR_FR=True, FR_FR=True,
#		RR_L_Fitness=True, FR_L_Fitness=True, L_L_Fitness=True, RR_RR_Fitness=True, 
#                RR_FR_Fitness=True, FR_FR_Fitness=True
#6 possible scorers:
#case1: rec-ligand
#CREATE SPECIFIED molecular systems:
#L_L (1)    T   vs dlgs w/flexres    w/out flexRes
#RR_RR (2)  T   @@ ==>False         False
#FR_FR (3)  T         True      ==> False
#FR_L (4)   T         True      ==> False
#RR_L (5)   T         True
#RR_FR (6)  T         True      ==> False
# RR_L, FR_L, RR_FR <=Intermolecular energies
#@@ *_Fitness internal energy 
#INCLUDE score in GA Fitness
#RR_L_Fitness  T 
#FR_L_Fitness  T
#L_L_Fitness   T @@
#RR_RR_Fitness T     False         False
#RR_FR_Fitness T     True          False
#FR_FR_Fitness T @@  True          True



#check these autodock comparisons:
#case1: rec-ligand
#case2: rec-ligand-flexres

if __name__ == '__main__':
    import sys
    import getopt


# compute_AD42ScoreC.py -r receptorfilename -l ligandfilename [-o outputfilename -v
# for comparison with docking output: these are set to False: RR_RR, 
    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: compute_AD42ScoreC.py -r receptorfilename -l ligandfilename"
        print
        print "    Description of command..."
        print "      Calculates the AD42ScoreC energy between two molecules and stores it in 'AD42ScoreC_scores.txt'"
        print "      By default...."
        print "         -r     receptorfilename"
        print "         -l     ligandfilename"
        print "    Optional parameters:"
        print "        [-o]    outputfilename (default is 'AD42ScoreC_scores.txt')"
        print "        [-x]    flexresfilename (default is none)"
        print "        [-w]    write_file_mode (default mode is 'a')"
        print "        [-p]    printAllScoreTerms (default is not to print them all)"
#defaults based on assumption using simplest case: receptor-ligand
#        print "        [-R]    include Rigid-Rec - Rigid-Rec (default is False)"
#        print "        [-I]    include Rigid-Rec - Flex-Rec (default is False)"
#        print "        [-f]    include FR_L_Fitness (default is False)"
#        print "        [-g]    include RR_RR_Fitness (default is True)"
#        print "        [-j]    include RR_FR_Fitness (default is False)"
#        print "        [-k]    include FR_FR_Fitness (default is False)"
#        print "        [-m]    flexRec Atoms  (default is None)"
#        print "        [-p]    printAllScoreTerms (default is False)"
        print "        [-v]    verbose output"

    FR_L = False  #no Flex-Rec - Ligand @@ True by default in AD42ScoreC
    L_L = True    # internal energy ligand
    RR_RR = False  # internal energy Rigid-Rec
    RR_L = True   # rigid-receptor ligand
    # if flexresfilename, turn these on: RR_FR, FR_FR, FR_L_Fitness, 
    RR_FR = False #no Rigid-Rec - Flex-Rec @@ True by default
    FR_FR = False  #no Flex-Rec - Flex-Rec @@ True by default
    RR_L_Fitness = True #Rigid-Rec - Ligand True by default
    L_L_Fitness = True  # ?
    FR_L_Fitness = False #no Flex-Rec - Ligand score @@ True by default
    RR_RR_Fitness = False #Rigid-Rec - Rigid-Rec Fitness
    RR_FR_Fitness = False #no Rigid-Rec - Flex-Rec Fitness @@ True by default
    FR_FR_Fitness = False #no Flex-Rec - Flex-Rec score @@ True by default
    cutoff = 1.00

    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'r:l:o:x:w:pvh')
        #opt_list, args = getopt.getopt(sys.argv[1:], 'r:l:o:x:wzRIfgjkmvh')
    except getopt.GetoptError, msg:
        print 'compute_AD42ScoreC.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-r: receptorfilename
    receptorfilename =  None
    #-l: ligandfilename
    ligandfilename =  None
    # optional parameters
    #-o outputfilename
    outputfilename = 'AD42ScoreC_scores.txt'
    #-x flexresfilename 
    flexresfilename = None
    flexRecAtoms = None
    FR_L_Fitness = False
    #-w write new outputfile
    write_file_mode = 'a'
    #FR_L = False
    #RR_FR = False
    #-R include_RigidRec_RigidRec #@@ july3 sanity
    #RR_RR = False #@@True by default
    #-I include_RigidRec_FlexRec 
    #RR_FR = False
    #-f include_FR_L_Fitness
    #-g include_RR_RR_Fitness 
    #RR_RR_Fitness = False
    #-j include_RR_FR_Fitness 
    #RR_FR_Fitness = False
    #-k include_FR_FR_Fitness 
    #FR_FR_Fitness = False
    #-p printAllScoreTerms 
    printAllScoreTerms = False
    #-v verbose
    verbose = None
    # current args:
    # r:l:o:x:w:pvh
    # previously:
    #'r:l:o:x:w:RIfgjkmpvh'

    for o, a in opt_list: #r:l:o:x:w:pvh
        if o in ('-r', '--r'):
            receptorfilename = a
            if verbose: print 'set receptorfilename to ', a
        if o in ('-l', '--l'):
            ligandfilename = a
            if verbose: print 'set ligandfilename to ', a
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-x', '--x'):
            flexresfilename = str(a)
            print "line 148: a=", a
            if verbose: print 'set flexresfilename to ', flexresfilename
            flexres = Read(flexresfilename)[0]
            flexres.buildBondsByDistance()
            flexRecAtoms = flexres.allAtoms
            #print "now flexRecAtoms=", flexRecAtoms
            FR_L = True
            FR_L_Fitness= True
            #RR_FR = True awaiting fix of  BUG in AutoDockFR/ScoringFunction.py => recIE never initialized
            #FR_FR = True #@@ internal energy of flexres
            ##RR_FR_Fitness = True #1
            ##FR_FR_Fitness = True #2
        if o in ('-w', '--w'):
            write_file_mode = str(a)
            if verbose: print 'set write_file_mode to ', a
        if o in ('-p', '--p'):
            printAllScoreTerms = True
            if verbose: print "set printAllScoreTerms to ", printAllScoreTerms
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if not receptorfilename:
        print 'compute_AD42ScoreC: receptorfilename must be specified.'
        usage()
        sys.exit()

    if not ligandfilename:
        print 'compute_AD42ScoreC: ligandfilename must be specified.'
        usage()
        sys.exit()

    ##@@ L_L = True  #@@ ==  AD42ScoreC default: Ligand - Ligand
#    RR_L = True  #@@ == AD42ScoreC default: Rigid-Rec - Ligand
#    FR_L = False  #no Flex-Rec - Ligand @first pass: assume no flexres
#    RR_RR = False  #@@    Rigid-Rec - Rigid-Rec  internal energy of receptor :o 
#    RR_FR = False #no Rigid-Rec - Flex-Rec @first pass
#    FR_FR = False #no Flex-Rec - Flex-Rec  @first pass
#    FR_L_Fitness = False #no Flex-Rec - Ligand score
#    RR_L_Fitness = True #@@ 
#    L_L_Fitness = True #@@
#    RR_RR_Fitness = False #@@
#    RR_FR_Fitness = False #no Rigid-Rec - Flex-Rec score
#    FR_FR_Fitness = False #no Flex-Rec - Flex-Rec score
 

    receptor = Read(receptorfilename)[0]
    assert os.path.splitext(receptor.parser.filename)[-1]=='.pdbqt',"receptor file not in required '.pdbqt' format"
    if verbose: print 'read ', receptorfilename
    receptor.buildBondsByDistance()
    #check format of receptor ?@@?
    ligand = Read(ligandfilename)[0]
    assert os.path.splitext(ligand.parser.filename)[-1]=='.pdbqt',"ligand file not in required '.pdbqt' format"
    if verbose: print 'read ', ligandfilename
    ligand.buildBondsByDistance()    

    ad42sc = AD42ScoreC(receptor.allAtoms, ligand.allAtoms, ligand.torTree, ligand.TORSDOF,
                        flexRecAtoms=flexRecAtoms, RR_L=RR_L, FR_L=FR_L, RR_FR=RR_FR, RR_RR=RR_RR, FR_FR=FR_FR,
                        FR_L_Fitness=FR_L_Fitness, FR_FR_Fitness=FR_FR_Fitness, RR_RR_Fitness=RR_RR_Fitness,
                        RR_FR_Fitness=RR_FR_Fitness)

    # compute the score...
    score = ad42sc.score()
    print "score=", score

    if printAllScoreTerms:
        #print "calling ad42sc.printAllScoreTerms"
        #print dir(ad42sc)
        ad42sc.printAllScoreTerms()
    mode = 'a'
    first = not os.path.exists(outputfilename)
    if write_file_mode:
        mode = 'w'
        first = True
    if verbose: print 'first is ', first
    optr = open(outputfilename, mode)
    if first:
        tstr = "    Receptor      Ligand   AD42Score\n" 
        optr.write(tstr)
    #setup the scorer:
    ostr = "%12s-%12s      %6.3f\n" %(receptor.name,ligand.name, score)
    optr.write(ostr)
    if verbose:
        print 'wrote %s' %outputfilename
    optr.close()

# To execute this command type:
# compute_AD42ScoreC.py -r receptorfilename -l ligandfilename [-o outputfilename  -w write_file_mode] -v







#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/prepare_flexreceptor4.py,v 1.8 2014/04/01 17:54:21 rhuey Exp $
#
import os 

from MolKit import Read
from MolKit.protein import ProteinSet, ResidueSet
from MolKit.molecule import BondSet
from MolKit.stringSelector import CompoundStringSelector

from AutoDockTools.MoleculePreparation import AD4FlexibleReceptorPreparation



if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: prepare_flexreceptor4.py -r receptor_filename -s list_of_names_of_residues_to_move"
        print "    Description of command..."
        print "         -r     receptor_filename (.pdbqt)"
        print "         -s     specification for flex residues" 
        print "                Use underscores to separate residue names:"
        print "                  ARG8_ILE84  "
        print "                Use commas to separate 'full names' which uniquely identify residues:"
        print "                  hsg1:A:ARG8_ILE84,hsg1:B:THR4 "
        print "                [syntax is molname:chainid:resname]"
        print "    Optional parameters:"
        print "        [-v]    verbose output"
        print "        [-N]    type(s) of bonds to disallow: "
        print "        [-P]    pairs of atom names bonds between which to disallow: "
        print "        [-g pdbqt_filename] (rigid output filename)"
        print "        [-x pdbqt_filename] (flexible output filename)"

    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'r:vs:N:P:g:x:h')
    except getopt.GetoptError, msg:
        print 'prepare_flexreceptor4.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-r: receptor
    receptor_filename =  None
    #-s: residues_to_move
    residues_to_move =  None
    # optional parameters
    verbose = None
    #-N: type of bonds to  disallow
    disallow = ""
    #-P: pairs of atom names bonds between which to  disallow
    disallowed_pairs = ""
    #-g  : rigid output filename
    rigid_filename = None
    #-x  : flexible output filename
    flexres_filename = None

    #'r:vs:N:g:x:h'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-r', '--r'):
            receptor_filename = a
            if verbose: print 'set receptor_filename to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to', True
        if o in ('-s', '--s'):
            residues_to_move = a
            if verbose: print 'set residues_to_move to ', a
        if o in ('-N', '--N'):
            disallow = a
            if verbose: print 'set disallow to ', a
        if o in ('-P', '--P'):
            disallowed_pairs = a
            if verbose: print 'set disallowed_pairs to ', a
        if o in ('-g', '--g'):
            rigid_filename = a
            if verbose: print 'set rigid_filename to ', a
        if o in ('-x', '--'):
            flexres_filename = a
            if verbose: print 'set flexres_filename to ', a
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if not  receptor_filename:
        print 'prepare_flexreceptor4: receptor filename must be specified!\n'
        usage()
        sys.exit()

    if not  residues_to_move:
        print 'prepare_flexreceptor4: residues to move must be specified!\n'
        usage()
        sys.exit()

    extension = os.path.splitext(receptor_filename)[1]
    if extension!=".pdbqt":
        print 'prepare_flexreceptor4: receptor file must be in .pdbqt format\n'
        usage()
        sys.exit()

    r = Read(receptor_filename)[0]
    r.buildBondsByDistance()
    if verbose: print 'read ', receptor_filename

    all_res = ResidueSet()
    # hsg1:A:ARG8_ILE82;hsg1:B:THR4 
    # ARG8_ILE84 
    names_by_chain = residues_to_move.split(',')
    if verbose: 
        print "Specified flexres selection strings are:"
        for strN in names_by_chain: 
            print "   %s" %strN
    #1. ['hsg1:A:ARG8_ILE82','hsg1:B:THR4'] 
    # OR 
    #2. ARG8_ILE84
    for res_names_by_chain in names_by_chain:
        #check for ':'
        if verbose: print "selecting ", res_names_by_chain
        if res_names_by_chain.find(':')==-1:
            # no ':' in selection string = simple case eg: ARG8_ILE84 
            res_names = res_names_by_chain.split('_')
            if verbose: print "res_names=", res_names
            for n in res_names:
                if verbose: print "looking for  ",n
                res = r.chains.residues.get(lambda x: x.name==n)
                if len(res):
                    all_res += res
                    if verbose: print " added ", res.name, " to ", all_res
                else:
                    print "WARNING: no residue named " + n 
        else:
            # 'hsg1:A:ARG8_ILE82'
            # '1JFF_protein:A:THR179_GLU183,1JFF_protein:B:TRP21_ARG64_LEU275'
            chainStart = res_names_by_chain.index(':')+1
            molName = res_names_by_chain[:chainStart-1]
            #if verbose: print "molName = ", molName, " chainStart=", chainStart
            n = res_names_by_chain[chainStart:].replace("_", ",")
            #if verbose: print "after comma replaced _ n is ", n, '\n'
            selStr = molName + ":" + n
            #if verbose: print "now selStr", selStr
            # 'hsg1:A:ARG8,ILE82'
            #res, msg = CompoundStringSelector().select(ProteinSet([r]), n)
            res, msg = CompoundStringSelector().select(ProteinSet([r]), selStr)
            #if verbose: print selStr, " selection =", res, " msg=", msg, '\n'
            if len(res):
                all_res += res
            else:
                print "no residue found using string ", selStr
    #if verbose: print "built all_res=", all_res.full_name()
    #check for duplicates
    d = {}
    for res in all_res: d[res] = 1
    all_res = d.keys()
    all_res = ResidueSet(all_res).uniq()
    all_res.sort()
    if verbose: 
        print "located ", len(all_res),  " residues to format:"
        for z in all_res: print "   %s" %z.full_name()

    #inactivate specified bonds
    #disallowed_Pairs "CA_CB:CB_CG:C_CA"
    all_bnds = BondSet()
    #inactivate specified bonds
    #disallowed_pairs "CA_CB:CB_CG:C_CA"
    if len(disallowed_pairs):
        bnd_pairs = disallowed_pairs.split(':')
        for pair in bnd_pairs:
            names = pair.split('_')
            bnds = all_res.atoms.bonds[0].get(lambda x: x.atom1.name in names and x.atom2.name in names)
            if len(bnds):
                all_bnds += bnds
   
    if verbose:  print "beginning formatting ..."
    fdp = AD4FlexibleReceptorPreparation(r, residues=all_res, rigid_filename=rigid_filename, 
                                            flexres_filename=flexres_filename,
                                            non_rotatable_bonds=all_bnds)

    if verbose:  print "finished!"

# To execute this command type:
# prepare_flexreceptor4.py -r receptor_filename -s list_of_names_of_residues_to_move"







# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/bin/runAdt.py,v 1.33 2008/07/30 19:02:47 vareille Exp $
# $Id: runAdt.py,v 1.33 2008/07/30 19:02:47 vareille Exp $

# AutoDockTools can be launched from a python shell like this:
#import AutoDockTools; AutoDockTools.runADT()

import sys
import AutoDockTools

if '__IP' in dir(): # ipython
    ownInterpreter = False
else:
    ownInterpreter = True 

if '__file__' in locals():
    AutoDockTools.runADT(sys.argv, ownInterpreter=ownInterpreter, AdtScriptPath=__file__)
else:
    AutoDockTools.runADT(sys.argv, ownInterpreter=ownInterpreter)



#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/prepare_dpf42.py,v 1.16 2013/06/14 17:13:18 rhuey Exp $
#

import string
import os.path
from MolKit import Read
from AutoDockTools.DockingParameters import DockingParameters, genetic_algorithm_list4_2, \
                genetic_algorithm_local_search_list4_2, local_search_list4_2,\
                simulated_annealing_list4_2, epdb_list4_2,\
                DockingParameter42FileMaker

from AutoDockTools.atomTypeTools import AutoDock4_AtomTyper
from numpy import oldnumeric as Numeric
                

def usage():
    print "Usage: prepare_dpf42.py -l pdbqt_file -r pdbqt_file"
    print "Description of command..."
    print "Prepare a docking parameter file (DPF) for AutoDock42: <ligand>_<receptor>.dpf"
    print "  containing genetic_algorithm_local_search_list4_2 parameters (GALS)"
    print "    -l ligand_filename"
    print "    -r receptor_filename"
    print
    print "Optional parameters:"
    print "    [-o output dpf_filename]"
    print "    [-i template dpf_filename]"
    print "    [-x flexres_filename]"
    print "    [-p parameter_name=new_value]"
    print "    [-k list of parameters to write]"
    print "    [-e write epdb dpf ]"
    print "    [-v] verbose output"
    print "    [-L] use local search parameters"
    print "    [-S] use simulated annealing search parameters"
    print "    [-s] seed population using ligand's present conformation"
    print "    [-A] use only root atom coordinates to calculate about"
    print

    
if __name__ == '__main__':
    import getopt
    import sys

    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'sLShveAl:r:i:o:x:p:k:')
    except getopt.GetoptError, msg:
        print 'prepare_dpf42.py: %s' % msg
        usage()
        sys.exit(2)

    receptor_filename = ligand_filename = None
    dpf_filename = None
    template_filename = None
    flexres_filename = None
    parameters = []
    parameter_list = genetic_algorithm_local_search_list4_2
    pop_seed = False
    local_search = False
    verbose = None
    epdb_output = False
    about_root_atoms_only = False
    for o, a in opt_list:
        if verbose: print "o=", o, ' a=', a
        if o in ('-v', '--v'):
            verbose = 1
            if verbose: print 'verbose output'
        if o in ('-l', '--l'):   #ligand filename
            ligand_filename = a
            if verbose: print 'ligand_filename =', ligand_filename
        if o in ('-r', '--r'):   #receptor filename
            receptor_filename = a
            if verbose: print 'receptor_filename =', receptor_filename
        if o in ('-x', '--x'):   #flexres_filename 
            flexres_filename = a
            if verbose: print 'flexres_filename =', flexres_filename
        if o in ('-i', '--i'):   #input reference
            template_filename = a
            if verbose: print 'template_filename =', template_filename
        if o in ('-o', '--o'):   #output filename
            dpf_filename = a
            if verbose: print 'output dpf_filename =', dpf_filename
        if o in ('-p', '--p'):   #parameter
            parameters.append(a)
            if verbose: print 'parameters =', parameters
        if o in ('-e', '--e'):
            epdb_output = True
            if verbose: print 'output epdb file'
            parameter_list = epdb_list4_2
        if o in ('-k', '--k'):   #parameter_list_to_write
            parameter_list = a
            if verbose: print 'parameter_list =', parameter_list
        if o in ('-L', '--L'):   #parameter_list_to_write
            local_search = 1
            parameter_list = local_search_list4_2
            if verbose: print 'parameter_list =', parameter_list
        if o in ('-S', '--S'):   #parameter_list_to_write
            parameter_list = simulated_annealing_list4_2
            if verbose: print 'parameter_list =', parameter_list
        if o in ('-A', '--A'):   #set about to average of coords of root atoms
            about_root_atoms_only = True
            if verbose: print 'about_root_atoms_only =', about_root_atoms_only
        if o in ('-h', '--'):
            usage()
            sys.exit()
        if o in ('-s', '--s'):
            pop_seed = True
           


    if (not receptor_filename) or (not ligand_filename):
        print "prepare_dpf42.py: ligand and receptor filenames"
        print "                    must be specified."
        usage()
        sys.exit()


    #11/2011: fixing local_search bugs:
    # specifically: 
    # 1. quaternion0 0 0 0 0  
    # 2. dihe0 0 0 0 0 0 <one per rotatable bond>
    # 3. about == tran0 
    # 4. remove tstep  qstep and dstep
    # 5. remove ls_search_freq
    #local_search = local_search_list4_2
    #parameter_list =local_search_list4_2
    dm = DockingParameter42FileMaker(verbose=verbose, pop_seed=pop_seed)
    if template_filename is not None:  #setup values by reading dpf
        dm.dpo.read(template_filename)
    dm.set_ligand(ligand_filename)
    dm.set_receptor(receptor_filename)
    if flexres_filename is not None:
        flexmol = Read(flexres_filename)[0]
        flexres_types = flexmol.allAtoms.autodock_element
        lig_types = dm.dpo['ligand_types']['value'].split()
        all_types = lig_types
        for t in flexres_types:
            if t not in all_types: 
                all_types.append(t)
        all_types_string = all_types[0]
        if len(all_types)>1:
            for t in all_types[1:]:
                all_types_string = all_types_string + " " + t
                if verbose: print "adding ", t, " to all_types->", all_types_string
        dm.dpo['ligand_types']['value'] = all_types_string 
        dm.dpo['flexres']['value'] = flexres_filename
        dm.dpo['flexres_flag']['value'] = True
    #dm.set_docking_parameters( ga_num_evals=1750000,ga_pop_size=150, ga_run=20, rmstol=2.0)
    kw = {}
    for p in parameters:
        key,newvalue = string.split(p, '=')
        #detect string reps of lists: eg "[1.,1.,1.]"
        if key=='parameter_file':
            if key in parameter_list:
                print "removing parameter_file keyword"
                parameter_list.remove('parameter_file')
            parameter_list.insert(1, key)
            dm.dpo['custom_parameter_file']['value']=1
        if newvalue[0]=='[':
            nv = []
            for item in newvalue[1:-1].split(','):
                nv.append(float(item))
            #print "nv=", nv
            newvalue = nv
            kw[key] = nv
            if verbose: print "newvalue=", nv, " kw=", kw
        elif key=='epdb_flag':
            if verbose: print "setting epdb_flag to", newvalue
            kw['epdb_flag'] = 1
        elif 'flag' in key:
            if verbose: print "key=", key, ' newvalue=', newvalue
            if newvalue in ['1','0']:
                newvalue = int(newvalue)
            if newvalue =='False':
                newvalue = False
            if newvalue =='True':
                newvalue = True
        elif local_search and 'about' in key:
            kw['about'] = newvalue
            kw['tran0'] = newvalue     
        else:         
            kw[key] = newvalue
            if verbose: print "set ", key, " to ", newvalue
        if verbose: print "calling set_docking_parameters with kw=", kw
        apply(dm.set_docking_parameters, (), kw)
        if key not in parameter_list:
            #special hacks for output_pop_file,set_sw1,...
            if key=='output_pop_file':
                parameter_list.insert(parameter_list.index('set_ga'), key)
            elif key=='set_sw1':
                parameter_list.insert(parameter_list.index('ls_search_freq')+1, key)
            else:
                parameter_list.append(key) 
    if about_root_atoms_only:
        lines = dm.ligand.parser.allLines
        for ix, lll in enumerate(lines):
            if lll.find("ROOT")==0: 
                root_ix = ix
            if lll.find("ENDROOT")==0: 
                endroot_ix = ix
                break
        last_ix = endroot_ix - (root_ix + 1) #47-(18+1)
        crds = dm.ligand.allAtoms[0:last_ix].coords
        about = (Numeric.add.reduce(crds)/float(len(crds))).tolist()
        dm.dpo['about']['value'] = (round(about[0],3), round(about[1],3), round(about[2],3))
    if epdb_output:
        dm.dpo['epdb_flag']['value'] = 1
        dm.write_dpf(dpf_filename, parm_list=epdb_list4_2)
    else:
        if verbose: print "not epdb_output"
        dm.write_dpf(dpf_filename, parameter_list, pop_seed=pop_seed)

    
#prepare_dpf42.py -l indinavir.pdbqt -r 1hsg.pdbqt -p ga_num_evals=25000000 -p ga_pop_size=150 -p ga_run=17 -i ref.dpf -o testing.dpf 




cmdlist = [
("AutoDockTools","autoanalyzeCommands","ADanalyze_readDLG","""allows the user to select a filename for the docking log"""),
("AutoDockTools","autoanalyzeCommands","ADanalyze_deleteDLG","""allows the user to delete docking """),
("AutoDockTools","autoanalyzeCommands","ADanalyze_selectDLG","""allows the user to change current docking """),
("AutoDockTools","autoanalyzeCommands","ADanalyze_chooseDockedConformations",""""""),
("AutoDockTools","autoanalyzeCommands","ADanalyze_readMacromolecule","""this class is used to load the macromolecule specified in the docking log"""),
#("AutoDockTools","autoanalyzeCommands","ADanalyze_showHistogram",""""""),
("AutoDockTools","autoanalyzeCommands","ADanalyze_readAllDLGInDirectory","""allows the user to read all the docking logs in a chosen directory"""),
#("AutoDockTools","autoanalyzeCommands","ADanalyze_getChart",""""""),
#("AutoDockTools","autoanalyzeCommands","ADanalyze_showResultsOutput","""When the docking log is parsed, summary lines are detected and saved in  """),
("AutoDockTools","autoanalyzeCommands","ADanalyze_showGridIsocontours",""" toplevel GUI with one row per grid_type allowing: \n            loading a grid (first, then->),  \n            toggling the visiblity of its isosurface,\n            changing sampling rate, \n            changing isocontour isovalue,\n            changing the render mode between LINE and FILL\n            visibility of the grid's bounding box\n    """),
("AutoDockTools","autoanalyzeCommands","ADanalyze_addExtraGridIsocontour",""" adds a row to the toplevel GUI of ADMakeAllGrids enabling:\n            loading a USER grid (first, then->),  \n            toggling its visiblity,\n            changing sampling rate, \n            changing isocontour value,\n            changing the render mode between lines and fill\n    """),
("AutoDockTools","autodpfCommands","ADdpf_setDpo",""""""),
("AutoDockTools","autodpfCommands","ADdpf_read",""" allows user to select a file containing a set of defaults"""),
("AutoDockTools","autodpfCommands","ADdpf_chooseMacromolecule",""" allows user to choose a molecule already present for the macromolecule"""),
("AutoDockTools","autodpfCommands","ADdpf_readMacromolecule",""" allows user to select a filename for the macromolecule"""),
("AutoDockTools","autodpfCommands","ADdpf_chooseFormattedLigand",""" allows user to choose a molecule already present for the ligand"""),
("AutoDockTools","autodpfCommands","ADdpf_readFormattedLigand",""" allows user to choose a PDBQ file for the ligand"""),
("AutoDockTools","autodpfCommands","ADdpf_initLigand","""initializes the ligand"""),
("AutoDockTools","autodpfCommands","ADdpf_setSAparameters",""" allows user to set necessary parameters for simulated annealing-based autodock job"""),
("AutoDockTools","autodpfCommands","ADdpf_setGAparameters",""" allows user to set necessary parameters for genetic algorithm-based autodock job"""),
("AutoDockTools","autodpfCommands","ADdpf_setLSparameters",""" allows user to set necessary parameters for local search-based autodock job"""),
("AutoDockTools","autodpfCommands","ADdpf_setDockingParameters",""" allows user to set these parameters for  autodock job: step sizes, energy parameters and format for the output"""),
("AutoDockTools","autodpfCommands","ADdpf_writeSA",""" allows user to choose an output filename and write simulated annealing parameters"""),
("AutoDockTools","autodpfCommands","ADdpf_writeGA",""" allows user to choose an output filename and write it"""),
("AutoDockTools","autodpfCommands","ADdpf_writeLS",""" allows user to choose an output filename and write it"""),
("AutoDockTools","autodpfCommands","ADdpf_writeGALS",""" allows user to choose an output filename and write it"""),
("AutoDockTools","autodpfCommands","ADdpf_writeCluster",""" allows user to write a dpf to cluster many dockings """),
("AutoDockTools","autodpfCommands","ADdpf_edit",""" allows user to edit current output file and write it"""),
("AutoDockTools","autogpfCommands","ADgpf_setGpo",""" Command to set values in gpo object using gui input results"""),
("AutoDockTools","autogpfCommands","ADgpf_initLigand","""initializes the ligand:\n        checkCharges\n        getTypes\n        getSideLengths\n        setAutoDockElements\n        colorByAtom + color aromatic C's green\n        \n        GUI allows user to:\n            decide which types of hydrogen bonds to model\n            modify list of atom types detected in ligand\n        \n        """),
("AutoDockTools","autogpfCommands","ADgpf_initMacro","""receptor is initialized as specified in parameters:\n            autodock_element field added\n            charges may be added (either Kollman or Gasteiger)\n            charges are checked\n            lone pairs merged\n            non-polarhydrogens merged\n            macro sidelengths set\n            types of atoms in receptor added as '.mset'\n                possibility of adding new atom types to Rij/epsij dicts\n            solvation parameter fields are added: AtVol + AtSolPar\n                which is written to newfilename if specified \n                otherwise to mol.name+'.pdbqs'\n            receptor is colored by Atom Type"""),
("AutoDockTools","autogpfCommands","ADgpf_checkMacroTypes","""detect types of atoms in receptor; \n        \n       for non-standard types (limited to 2:'X' and 'M'):\n            define new energy parameters: \n                Rij and epsij\n            write file substituting X or M for non-standard element """),
("AutoDockTools","autogpfCommands","ADgpf_chooseMacromolecule",""" allows user to choose a molecule already present for the receptor"""),
("AutoDockTools","autogpfCommands","ADgpf_readMacromolecule",""" allows user to select the receptor via a file browser"""),
("AutoDockTools","autogpfCommands","ADgpf_mergeNonPolarHydrogens",""""""),
("AutoDockTools","autogpfCommands","ADgpf_addSolvationParameters",""" allows user to add solvation parameters to a receptor """),
("AutoDockTools","autogpfCommands","ADgpf_setMapTypes",""" allows user to enter types of maps to be calculated, directly"""),
("AutoDockTools","autogpfCommands","ADgpf_chooseFormattedLigand",""" allows user to choose a molecule already present for the ligand"""),
("AutoDockTools","autogpfCommands","ADgpf_readFormattedLigand",""" allows user to choose a ligand file via a file browser"""),
("AutoDockTools","autogpfCommands","ADgpf_defineAtomParameters",""""""),
("AutoDockTools","autogpfCommands","ADgpf_setUpCovalentMap",""""""),
("AutoDockTools","autogpfCommands","ADgpf_setGrid",""" allows user to set center, spacing, and extent of \n        grids (number of pts in each dimension) to be calculated"""),
("AutoDockTools","autogpfCommands","ADgpf_setOtherOptions",""" allows user to set smooth factor, whether to calculate\n        floating point and electrostatic maps AND whether to \n        use a constant or a distance-dependent dielectric constant"""),
("AutoDockTools","autogpfCommands","ADgpf_writeGPF",""" allows user to choose an output filename and write it"""),
("AutoDockTools","autogpfCommands","ADgpf_editGPF",""" allows user to edit current output file and write it"""),
("AutoDockTools","autogpfCommands","ADgpf_readGPF",""" allows user to select a file containing a set of defaults"""),
("AutoDockTools","autogpfCommands","ADgpf_selectCenter",""" allows user to pick an atom to be the center of the grid"""),
("AutoDockTools","autostartCommands","ADstart_autogrid","""Interactive usage: \n            The user chooses host and parameter file and starts the Autogrid job. \n            If the host has an interactive queue, launching the job opens \n            a 'ADstart_manage' widget which allows the user to follow the job \n            and to kill it, if necesary. A 'job file' is written when a parameter \n            file is selected in combination with the selection of a 'nqe'-type host.\n        Scripting usage:\n            'doIntRemoteCommand' and 'doNqeRemoteCommand' methods allow starting \n            AutoGrid with the specified parameter file on a host with\n            a appropriate queue type. (if not specified, host is assumed\n            to be the local host and must be of appropriate queue type).\n            All other parameters are optional"""),
("AutoDockTools","autostartCommands","ADstart_autodock","""Interactive usage: \n            The user chooses host and parameter file and starts the Autodock job. \n            If the host has an interactive queue, launching the job opens \n            a 'ADstart_manage' widget which allows the user to follow the job \n            and to kill it, if necesary. A 'job file' is written when a parameter \n            file is selected in combination with the selection of a 'nqe'-type host.\n        Scripting usage:\n            'doIntRemoteCommand' and 'doNqeRemoteCommand' methods allow starting \n            Autodock with the specified parameter file on a host with\n            a appropriate queue type. (if not specified, host is assumed\n            to be the local host and must be of appropriate queue type).\n            All other parameters are optional"""),
("AutoDockTools","autostartCommands","ADstart_manage",""""""),
("AutoDockTools","autostartCommands","ADstart_editHostMacros",""" this class allows user to add entries to hosts dictionary \n        and write them to a file"""),
("AutoDockTools","autotorsCommands","ADtors_readLigand","""allows user to select a file for the ligand via a file browser"""),
("AutoDockTools","autotorsCommands","ADtors_chooseLigand","""allows user to choose as ligand  a molecule already in the viewer"""),
("AutoDockTools","autotorsCommands","ADtors_rigidLigand","""allows user to write molecule with ROOT, ENDROOT + TORSDOF 0 added"""),
("AutoDockTools","autotorsCommands","ADtors_setRoot","""allows user to pick an atom to be ROOT, the rigid portion \n    of ligand which has rotatable BRANCHES """),
("AutoDockTools","autotorsCommands","ADtors_autoRoot","""causes program to pick an atom to be ROOT: the one which has\nthe smallest 'largest sub-tree'"""),
("AutoDockTools","autotorsCommands","ADtors_markRoot","""shows current extent of root portion of the molecule:it includes all contiguous atoms starting with those adjacent to the designated root atom out to first active torsion """),
("AutoDockTools","autotorsCommands","ADtors_defineRotBonds",""" """),
("AutoDockTools","autotorsCommands","ADtors_setBondRotatableFlag","""set the flag that tells whether a bond is rotatable in aan AutoDock\n    ligand"""),
#("AutoDockTools","autotorsCommands","ADtors_changePlanarCarbonsToA","""checks rings for planarity and changes carbons in planar rings to\nelement type and name: 'A'"""),
#("AutoDockTools","autotorsCommands","ADtors_changeAromaticCarbonsToC","""restores carbons in aromatic rings to element type 'C' and name: 'C...'"""),
("AutoDockTools","autotorsCommands","ADtors_setCarbonNames","""toggles Carbon names between 'A' + 'C'"""),
("AutoDockTools","autotorsCommands","ADtors_changePlanarityCriteria","""allows user to change requirement for aromaticity of cycles:\ndefault is < 5 degrees between normals to adjacent atoms.  User enters a new angle in degrees"""),
("AutoDockTools","autotorsCommands","ADtors_mergeNonPolarHydrogens","""merges non-polar hydrogen atoms w/ their carbons"""),
("AutoDockTools","autotorsCommands","ADtors_restoreMergedNonPolarHydrogens","""restores non-polar hydrogen atoms and their charges"""),
("AutoDockTools","autotorsCommands","ADtors_writeFormattedPDBQ","""allows user to select and write an output file for the formatted ligand"""),
("AutoDockTools","autotorsCommands","ADtors_showRootSphere","""lets user toggle rootSph visibility"""),
("AutoDockTools","autotorsCommands","ADtors_automaticLigandFormatting","""preforms default actions on designated molecule"""),
]



#!/usr/bin/env python
# SETUP input files:
#1. In your local MGLToolsPckgs directory:
#-create a pdbqt file containing the docked ligand conformation for the energy-breakdown.
#( I saved mine as "ind4.1.1best.pdbqt")
#-copy the receptor file here, too ('hsg1.pdbqt')
#3. In your local MGLToolsPckgs directory, start this script with mgltoolspckgs pythonsh
# to output to terminal:
#../bin/pythonsh ./script_score_atoms_by_component.py -l ind4.1.1best.pdbqt -r hsg1.pdbqt 
# to save output to a file:
#../bin/pythonsh ./script_score_atoms_by_component.py -l ind4.1.1best.pdbqt -r hsg1.pdbqt -o output.txt
# add '-d' to include an energy breakdown grouped as in a dlg:
#../bin/pythonsh ./script_score_atoms_by_component.py -l ind4.1.1best.pdbqt -r hsg1.pdbqt -o output.txt -d


#imports
import os
from MolKit import Read
from PyAutoDock.AutoDockScorer import AutoDock41Scorer, AutoDockTermWeights41
from PyAutoDock.MolecularSystem import MolecularSystem
from numpy import oldnumeric as Numeric



if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: score_atoms_by_component.py -r receptor_file -l ligand_file"
        print
        print "    Description of command..."
        print "         -r     receptor_file"
        print "         -l     ligand_file"
        print "    Optional parameters:"
        print "        [-o]    output_file"
        print "                      (default is scores.txt)"
        print "        [-d]    also include grouping energies in dlg-format"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'r:l:o:dvh')
    except getopt.GetoptError, msg:
        print 'score_atoms_by_component.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    verbose = None
    #-r receptor_file
    receptor_file = None
    #-l ligand_file
    ligand_file = None
    #[-o output_file]
    output_file = None
    optr = None   
    #[-d group energies as in dlg 'Run']
    include_dlg_grouping = False
                                                                                                                                 
    #'r:l:o:dvh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-o', '--o'):
            output_file = a
            try:
                optr = open(output_file, 'w')
            except:
                 print "problem opening file to contain per-atom energy breakdown:", output_file
                 sys.exit()
            if verbose: print 'set output_file to ', a
            optr = open(output_file, 'w')
        if o in ('-r', '--r'):
            receptor_file = a
            if not os.path.isfile(receptor_file):
                 print "receptor_file:", receptor_file, " does not exist!"
                 sys.exit()
            if verbose: print 'set receptor_file to ', a
        if o in ('-l', '--l'):
            ligand_file = a
            if not os.path.isfile(ligand_file):
                 print "ligand_file ", ligand_file, " does not exist!"
                 sys.exit()
            if verbose: print 'set ligand_file for energy component output to ', a
        if o in ('-d', '--d'):
            include_dlg_grouping = True
            if verbose: print 'set include_dlg_grouping to ', True
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if not receptor_file:
        print 'score_docked_atoms_by_component: receptor filename must be specified.'
        usage()
        sys.exit()

    if not ligand_file:
        print 'score_docked_atoms_by_component: ligand file must be specified.'
        usage()
        sys.exit()


    #1: create the python molecules
    rec = Read(receptor_file)[0]
    recbnds = rec.buildBondsByDistance()
    lig = Read(ligand_file)[0]
    ligbnds = lig.buildBondsByDistance()

    #2: setup python scorers for your molecules:
    ms = MolecularSystem()
    ms.add_entities(rec.allAtoms)
    ms.add_entities(lig.allAtoms)

    #3: setup python scorers
    scorer = AutoDock41Scorer()
    scorer.set_molecular_system(ms)
    estat_scores = Numeric.add.reduce(scorer.terms[0][0].get_score_array())
    dsolv_scores = Numeric.add.reduce(scorer.terms[1][0].get_score_array())
    vdw_scores = Numeric.add.reduce(scorer.terms[2][0].get_score_array())
    hbond_scores = Numeric.add.reduce(scorer.terms[3][0].get_score_array())
    wts = AutoDockTermWeights41()

    #4: open outputfile for writing scores
    if output_file is not None:
        fptr = open(output_file, 'w')

    #5. score the docked pose outputting individual terms for each atom:
    all_estat = []
    all_VHD = []
    for i in range(len(lig.allAtoms)):
        estat = round(estat_scores[i]*wts.estat_weight,3)
        vdw = round(vdw_scores[i]*wts.vdw_weight,3)
        hbond = round(hbond_scores[i]*wts.hbond_weight,3)
        dsolv = round(dsolv_scores[i]*wts.dsolv_weight,3)
        all_estat.append(estat)
        all_VHD.append(vdw+hbond+dsolv)
        ostr= "%4d:%6.3f %6.3f %6.3f %6.3f = %6.3f\n"%(i+1,estat,vdw,hbond,dsolv,estat+vdw+hbond+dsolv)
        if optr is None:
            if i==0: print "\n PyAutoDock AD4 energies:\n at#  estat    vdw  hbond  dsolv    total"
            print ostr[:-1]
        else:
            if i==0: optr.write( "\n PyAutoDock AD4 energies:\n at#  estat    vdw  hbond  dsolv    total\n")
            optr.write(ostr)

    #6. [optional: alternative grouping of output for comparison with numbers in dlg]
    #              output vdw+hb+ds ,estat for each atom:
    if include_dlg_grouping:
        for i in range(len(lig.allAtoms)):
            ostr ="%4d:%6.3f  %6.3f = %6.3f\n"%(i+1,all_estat[i], all_VHD[i], all_estat[i]+all_VHD[i])
            if optr is None:
                if i==0: print "\n energies grouped as in dlg:\n at#    vdW    Elec    total"
                print ostr[:-1]
            else:
                if i==0: optr.write( "\n energies grouped as in dlg:\n at#  ~vdW~    Elec    total\n")
                optr.write(ostr)
    # !Done!
    if optr is not None:
        optr.close()





#############################################################################
#
# Author: Ruth HUEY, Michel F. SANNER
#
# Copyright: M. Sanner TSRI 2008
#
#############################################################################


# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/autostart41Commands.py,v 1.5 2012/09/14 18:48:16 rhuey Exp $
#
# $Id: autostart41Commands.py,v 1.5 2012/09/14 18:48:16 rhuey Exp $ 
#
#
#
#
#
#

"""
This Module facilitates starting autogrid and autodock jobs and managing them

"""
from ViewerFramework.VFCommand import CommandGUI
from AutoDockTools.autostartCommands import ADKill, ADProcessManager,\
AutoStarter, AutoGridStarter, AutoDockStarter, AddAutoDockHost, removePCs,\
entropiaPresent, menuText

from AutoDockTools.autostartCommands import  VinaStarter


if entropiaPresent:
    from Entropia.EntropiaDef import entropia_job_dir
    from Entropia.EntropiaUI import EntropiaUI
    from Entropia.EntropiaEx import EntropiaError
    import ftplib

ADProcessManagerGUI=CommandGUI()
ADProcessManagerGUI.addMenuCommand('AutoTools41Bar', menuText['StartMB'], menuText['processManagerMB'])

AutoGridStarterGUI=CommandGUI()
AutoGridStarterGUI.addMenuCommand('AutoTools41Bar', menuText['StartMB'], menuText['startGridMB'])


AutoDockStarterGUI=CommandGUI()
AutoDockStarterGUI.addMenuCommand('AutoTools41Bar', menuText['StartMB'], menuText['startDockMB'])

VinaStarterGUI=CommandGUI()
VinaStarterGUI.addMenuCommand('AutoTools41Bar', menuText['StartMB'], menuText['startVinaMB'])
        
AddAutoDockHostGUI=CommandGUI()
AddAutoDockHostGUI.addMenuCommand('AutoTools41Bar', menuText['StartMB'], menuText['editHostsMB'])

commandList = [
    {'name':'AD41start_autogrid','cmd':AutoGridStarter(),'gui':AutoGridStarterGUI},
    {'name':'AD41start_autodock','cmd':AutoDockStarter(),'gui':AutoDockStarterGUI},
    {'name':'AD41start_vina','cmd':VinaStarter(),'gui':VinaStarterGUI},
    {'name':'ADstart_manage','cmd':ADProcessManager(),'gui':ADProcessManagerGUI},
    {'name':'AD41start_editHostMacros','cmd':AddAutoDockHost(),'gui':AddAutoDockHostGUI},
    ]

import sys, os
#if os.name != 'nt': #not sys.platform == 'win32':
#    commandList.insert(2,
#    {'name':'ADstart_manage','cmd':ADProcessManager(),'gui':ADProcessManagerGUI})
#else:
if os.name == 'nt': #not sys.platform == 'win32':
    import binaries
    os.environ['PATH'] = binaries.__path__[0]+";"+os.environ['PATH']
        
def initModule(vf):

    for dict in commandList:
        vf.addCommand(dict['cmd'],dict['name'],dict['gui'])

    if hasattr(vf,'GUI'):
        for item in vf.GUI.menuBars['AutoTools41Bar'].menubuttons.values():
            item.configure(background = 'tan')
            item.configure(underline = '-1')
    else:
        vf.addCommand(ADProcessManager(),'AD41start_manage')
        vf.addCommand(AutoGridStarter(), 'AD41start_autogrid')
        vf.addCommand(AutoDockStarter(), 'AD41start_autodock')
        vf.addCommand(VinaStarter(), 'AD41start_vina')



#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/summarize_wcg_docking.py,v 1.3 2007/10/08 18:14:32 rhuey Exp $
#
import os, glob, numpy.oldnumeric as Numeric
from string import split, join

from MolKit import Read
from MolKit.hydrogenBondBuilder import HydrogenBondBuilder
from AutoDockTools.XMLParser import XMLParser
from AutoDockTools.Docking import Docking
from AutoDockTools.cluster import Clusterer
from mglutil.math.rmsd import RMSDCalculator
from PyAutoDock.MolecularSystem import MolecularSystem
from PyAutoDock.AutoDockScorer import AutoDock4Scorer



if __name__ == '__main__':
    import sys
    import getopt

    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: summarize_wcg_docking.py -d directory"
        print
        print "    Description of command..."
        print "         -d     directory"
        print "    Optional parameters:"
        print "        [-t]    rmsd tolerance (default is 1.0)"
        print "        [-b]    print best docking info only (default is print all)"
        print "        [-o]    output filename"
        print "                      (default is directory.txt)"
        print "        [-a]    append to  output filename"
        print "                      (default is to open output filename 'w')"
        print "        [-k]    build hydrogen bonds"
        print "        [-r]    receptor filename"
        print "        [-e]    report energy breakdown"
        print "        [-v]    verbose output"
        print "                      (default is leave active)"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'd:o:t:r:bkaevh')
    except getopt.GetoptError, msg:
        print 'summarize_wcg_docking.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-d: directory
    directory =  None
    #-t: rms_tolerance
    rms_tolerance =  1.0
    # optional parameters
    verbose = None
    #-o outputfilename
    outputfilename = 'results.txt'
    #-r receptor_filename
    receptor_filename = ""
    #-b print best only
    print_best_only = False
    #-k build hydrogen bonds 
    build_hydrogen_bonds = False
    #-a append to outputfile
    append_to_outputfile = False
    #-e report_energy_breakdown 
    report_energy_breakdown = False


    #'d:o:t:r:bkaevh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-d', '--d'):
            directory = a   #faah001....
            if verbose: print 'set directory to ', a
            #CAUTION: THIS IS NOT GENERAL!!!
            #faah001_diversity0001_masterpr_ref...faah0001_xtpv_masterpr_ref
            ll = directory.split('_')
            #ll = ['faah0001', 'diversity0001', 'masterpr', 'ref']
            #OR
            #ll = ['faah0001', 'xtpv', 'masterpr', 'ref'']
            #eg faah0001
            key = ll[0]
            #diversity0001.pdbqt
            lig_fn = ll[1]+".pdbqt"
            #masterpr_ref.pdbqt
            receptor_filename = join(ll[2:],'_') + ".pdbqt"
            #dpf_fn 
            dpf_fn = directory  + ".dpf"
            print "dpf_fn =", dpf_fn
            #dpf_fn = directory + "/" + directory + ".dpf"
            #SET the default outputfilename to directory + .txt here
            if outputfilename=='results.txt':
                outputfilename = directory + '.txt'
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-t', '--t'):
            rms_tolerance = float(a)
            if verbose: print 'set rms_tolerance to ', a
        if o in ('-r', '--r'):
            #THIS IS SUPERFLUOUS
            receptor_filename = a
            print "warning! setting receptor_filename to ", a
            #if verbose: print 'set receptor_filename to ', a
        if o in ('-b', '--b'):
            print_best_only = True
            if verbose: print 'set print_best_only to ', True
        if o in ('-k', '--k'):
            build_hydrogen_bonds = True
            if verbose: print 'set build_hydrogen_bonds to ', True
        if o in ('-a', '--a'):
            append_to_outputfile = True
            if verbose: print 'set append_to_outputfile to ', True
        if o in ('-e', '--e'):
            report_energy_breakdown = True
            if verbose: print 'set report_energy_breakdown to ', True
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if not  directory:
        print 'summarize_wcg_docking: directory must be specified.'
        usage()
        sys.exit()

    #read all the xml docking logs in as one Docking
    
    xml_list = glob.glob('*.xml')
    print "xml_list =", xml_list
    p = XMLParser()
    d = Docking(parser=p)
    for xml_file in xml_list:
        print "calling readXMLResults with", xml_file
        d.readXMLResults(xml_file, dpf = dpf_fn)
    ligMol = d.ligMol
    ligAts = ligMol.allAtoms
    #setup rmsd tool
    coords = ligAts.coords[:]
    atom_ct = len(ligAts)
    torsion_ct = len(ligMol.torTree.torsionMap)
    tors_penalty = torsion_ct * 0.2744

    cl = Clusterer(d.ch.conformations)
    d.clusterer = cl
    cl.make_clustering(rms_tolerance)
    ref_coords = cl.clustering_dict[rms_tolerance][0][0].getCoords()[:]

    # for building hydrogen bonds or reporting energy breakdown
    # setup receptor:
    if build_hydrogen_bonds or report_energy_breakdown:
        ligMol.buildBondsByDistance()
        receptor = Read(receptor_filename)[0]
        receptor.buildBondsByDistance()
    if build_hydrogen_bonds:
        hbondBuilder = HydrogenBondBuilder()
    if report_energy_breakdown:
        ms = MolecularSystem()
        ms.add_entities(receptor.allAtoms)
        ms.add_entities(ligMol.allAtoms)
        adscorer = AutoDock4Scorer()
        adscorer.set_molecular_system(ms)

    mode = 'w'
    if append_to_outputfile:
        mode = 'a'
    fptr = open(outputfilename, mode)
    if mode=='w':
        if build_hydrogen_bonds and report_energy_breakdown:
            titles = "#clust  binding_nrg rmsd_v_0          Ki    int_nrg  hbnds sum_py_nrg estat_nrg     hbnd_nrg       vdw_nrg dsolv_nrg#ats #t\n"
        elif build_hydrogen_bonds:
            titles = "#clust  binding_nrg rmsd_v_0          Ki    int_nrg  hbnds #ats #t\n"
        elif report_energy_breakdown:
            titles = "#clust  binding_nrg rmsd_v_0          Ki    int_nrg  sum_py_nrg estat_nrg     hbnd_nrg       vdw_nrg dsolv_nrg#ats #t\n"
        else:
            titles = "#clust  binding_nrg rmsd_v_0          Ki    int_nrg #ats #t\n"
        fptr.write(titles)
    if print_best_only:
        clust0 = d.clusterer.clustering_dict[rms_tolerance][0]
        c = clust0[0]
        #find the filename of the best result
        #for k in d.dlo_list:
        #    if c in k.conformations:
        #        dlo_filename = os.path.splitext(k.parser.filename)[0]
        #        print "set dlo_filename to ", dlo_filename
        #        ostr = dlo_filename + ", "
        #        fptr.write(ostr)
        score_list = []
        #update the coords if you need to
        if build_hydrogen_bonds or report_energy_breakdown:
            d.ch.set_conformation(c)
        if report_energy_breakdown:
            ms.set_coords(1, c.getCoords()[:])
            score_list = adscorer.get_score_per_term()
            total_score = Numeric.add.reduce(score_list) + c.internal_energy + tors_penalty
            #total_score = Numeric.add.reduce(score_list)
        ostr = "% 4d, % 4d, % 12.4f, % 6.4f, % 8.6e,% 9.6f" %(len(d.clusterer.data), len(clust0),  c.binding_energy, c.getRMSD(ref_coords), c.Ki, c.internal_energy)
        if build_hydrogen_bonds:
            atDict = hbondBuilder.build(receptor.allAtoms, ligMol.allAtoms)
            num_hbonds = len(atDict)
            ostr = ostr + ", % 2d" %(num_hbonds)
        if report_energy_breakdown:
            ostr = ostr + ", % 12.4f" %total_score
            #for score in score_list:
            #    ostr = ostr + ", % 12.4f" %score
            ostr = ostr + ", % 6.4f, % 12.4f, % 12.4f,% 6.4f," %(score_list[0],
                        score_list[1], score_list[2], score_list[3])
        #add the number of atoms and the number of active torsions in the ligand
        ostr = ostr + "%4d%2d\n" %(atom_ct, torsion_ct)
        try:
            fptr.write(ostr)
        except:
            fptr.write("\n")
    else:
        for clust in d.clusterer.clustering_dict[rms_tolerance]:
            #for each cluster in the clustering at this rms_tolerance:
            #output length of cluster and energies of the lowest energy result
            c = clust[0]
            #write the dlg filename for the best in this cluster
            #for k in d.dlo_list:
            #    if c in k.conformations:
            #        dlo_filename = os.path.splitext(k.parser.filename)[0]
            #        print "dlo_filename=", dlo_filename
            #        #ostr = dlo_filename + "\n " 
            #        #fptr.write(ostr)
            score_list = []
            if build_hydrogen_bonds or report_energy_breakdown:
                d.ch.set_conformation(c)
            if report_energy_breakdown:
                ms.set_coords(1, c.getCoords()[:])
                score_list = adscorer.get_score_per_term()
                total_score = Numeric.add.reduce(score_list) + c.internal_energy + tors_penalty
            ostr = "% 4d, % 12.4f, % 8.4f, % 8.3e,% 9.6f" %(len(clust),  c.binding_energy, c.getRMSD(ref_coords), c.Ki, c.internal_energy)
            if build_hydrogen_bonds:
                atDict = hbondBuilder.build(receptor.allAtoms, ligMol.allAtoms)
                num_hbonds = len(atDict)
                ostr = ostr + ", % 2d" %(num_hbonds)
            if report_energy_breakdown:
                ostr = ostr + ", % 12.4f" %total_score
                ostr = ostr + ", % 6.4f, % 12.4f, % 12.4f,% 6.4f," %(score_list[0],
                            score_list[1], score_list[2], score_list[3])
                #for score in score_list:
                    #ostr = ostr + ",% 12.4f" %score
            ostr = ostr + "%3d %2d\n" %(atom_ct, torsion_ct)
            fptr.write(ostr)
    fptr.close()

    

# To execute this command type:
# summarize_wcg_docking.py -d directory -t rmsd tolerance -b report best docking
# only -r receptor filename  -k build hydrogen bonds -v




## Automatically adapted for numpy.oldnumeric Jul 23, 2007 by 

#############################################################################
#
# Author: Ruth HUEY, William  LINDSTROM
#
# Copyright: M. Sanner TSRI 2003
#
#############################################################################


#
#
# $Id: LigandMixin.py,v 1.4 2008/11/21 20:02:01 vareille Exp $
#
#
#
#
#
#
#

"""
This MIXIN Object adds AutoDock Ligand features to a MolKit.Protein
   methods:
       setup
           parameters:
               flags:
                useProteinAromaticList 
                autoMergeNPHS
               maxtors = 32
               aromaticCutOff = 7.5
           creates fields:
               bondClassifier 
               torsionTree = None
               message = ""
               torscount = 0
               possible_torscount = 0
               TORSDOF = 0
               #should this be here?
               aromaticCarbons = AtomSet()
           processes molecule:
               detects whether isPeptide
               checks charges
                   add charges if necessary
               checks hydrogens
                   merge nphs
                   warn if no polar hydrogens
               checks bonds, classifying them
                    via bondClassifier.classify:
               checks Aromatic carbons
       setroot
            self.ROOT
            #fix this: required by write method
            self.ROOT.rnum0 = 0
       autoroot
            self.autoRoot
            plus self.ROOT and self.ROOT.rnum0 = 0
       turnOnTorsion
       turnOffTorsion
       setCarbonName
       setAromaticName
       buildTorsionTree**
       write
       
   bondClassifier:
       classify:
           rotatablebonds
           amidebonds
           peptidebackbonebonds
           aromaticbonds (??)
               ???aromatic carbon detector???
               detect aromatic carbons
           sets torscount
           sets possible_torscount
           sets TORSDOF
           
   torsion tree which has:
       ROOT
       methods:
           set/auto root
           turn_off/turn_on torsions
               ->merge nodes
               ->split nodes
       ??RESIDUES??
       


"""
import numpy.oldnumeric as Numeric, math, types, os
from string import find
from MolKit.pdbWriter import PdbqWriter
from AutoDockTools.AutoDockBondClassifier import AutoDockBondClassifier
from AutoDockTools.observer import Subject
from MolKit.molecule import Atom, AtomSet, BondSet
from MolKit.protein import ProteinSet
from MolKit.torTree import TorTree
from Pmv.qkollua import q
from PyBabel.atomTypes import AtomHybridization
from PyBabel.bo import BondOrder
from PyBabel.cycle import RingFinder
from PyBabel.gasteiger import Gasteiger
from mglutil.util.misc import isInstance

aromDict = {}
aromDict['TRP'] = ['NE1','CD1','CG','CD2','CE2','CZ2',\
                    'CH2','CZ3','CE3']
aromDict['TYR'] = ['CD1','CG','CD2','CE1','CE2','CZ']
aromDict['PHE'] = ['CD1','CG','CD2','CE1','CE2','CZ']
aromDict['HIS'] = ['CD2','CE1','CG','ND1','NE2']
aromDict['PRO'] = ['N','CD','CA','CB','CG']
pep_aromList = ['PHE_CD1', 'PHE_CG', 'PHE_CD2', 'PHE_CE1',\
    'PHE_CE2', 'PHE_CZ', 'TYR_CD1', 'TYR_CG', 'TYR_CD2', 'TYR_CE1',\
    'TYR_CE2', 'TYR_CZ', 'HIS_CD2', 'HIS_CE1', 'HIS_CG', 'TRP_CD1',\
    'TRP_CG', 'TRP_CD2', 'TRP_CE2', 'TRP_CZ2', 'TRP_CH2', 'TRP_CZ3',\
    'TRP_CE3']

#MixIn methods from Chuck Esterbrook, 'Issue 84: Using Mix-ins with Python',
#                   Linux Journal Posted Sunday, April 01, 2001

def MixIn(pyClass, mixInClass, makeAncestor=0):
    if makeAncestor:
        if mixInClass not in pyClass.__bases__:
            pyClass.__bases__ = (mixInClass,) + pyClass.__bases__
    else:
        # Recursively traverse the mix-in ancestor
        # classes in order to support inheritance
        baseClasses = list(mixInClass.__bases__)
        baseClasses.reverse()
        for baseClass in baseClasses:
            MixIn(pyClass, baseClass)
        # Install the mix-in methods into the class
        for name in dir(mixInClass):
            if not name.startswith('__'):
                #skip private members
                member = getattr(mixInClass, name)
                if type(member) is types.MethodType:
                    member = member.im_func
                    setattr(pyClass, name, member)

                    
def SimpleMixIn(pyClass, mixInClass):
    if mixInClass not in pyClass.__bases__:
        pyClass.__bases__ += (mixInClass,)


class LigandMixin(Subject):
    """ lfo adds capability to a protein/molecule to be used as a ligand 
            in an AutoDock docking

    """


    def setup(self, useProteinAromaticList=1,
                    aromaticCutOff=7.5, maxtors=32,
                    autoMergeNPHS=1):
        if self.chains[0].hasBonds==0:
            self.buildBondsByDistance()
        self.useProteinAromaticList = useProteinAromaticList
        self.aromaticCutOff = aromaticCutOff
        self.maxtors = 32
        self.autoMergeNPHS = autoMergeNPHS
        self.bondClassifier = AutoDockBondClassifier()
        self.torsionTree = None
        self.message = ""
        self.torscount = 0
        self.possible_torscount = 0
        self.TORSDOF = 0
        self.resKeys = q.keys()
        self.PdbqWriter = PdbqWriter()
        #should this be here?
        msg = 'initialized ' + self.name +':\n'
        #detect whether isPeptide
        msg += self.checkIsPeptide()
        #process charges
        msg += self.checkCharges()
        ##process hydrogens
        msg += self.checkHydrogens()
        #process bonds
        #this doesn't report anything???
        newmsg, nphs = self.checkBonds()
        msg += newmsg
        #process aromaticCarbons
        msg += self.checkAromatics()
        return msg, nphs


    def checkIsPeptide(self):
        """
        checkIsPeptide
        """
        #check whether each restype is in std list
        #if so, self is a peptide
        resSet = self.chains.residues
        dict = {}
        for r in resSet:
            dict[r.type] = 0
        for t in dict.keys():
            if t not in self.resKeys:
                self.isPeptide = 0
                return '    -it is not a peptide\n'
        #only get to this point if all
        #residue types were found
        self.isPeptide = 1
        return '    -it is a peptide\n'


    def checkCharges(self):
        """
        checkCharges
        """
        msg = '     -already had charges'
        if hasattr(self, 'checked_charges'):
            return msg
        msg = ""
        needToAdd = 0
        chargedAts = self.allAtoms.get(lambda x: hasattr(x, 'charge'))
        if not chargedAts:
            needToAdd = 1
        elif len(chargedAts)!=len(self.allAtoms):
            needToAdd = 1
        elif len(filter(lambda x:x.charge==0, chargedAts))==len(chargedAts):
            #this checks that each atom doesn't have charge=0
            needToAdd = 1
        #to add Kollman need to:
        #   look up each atom's parent in q to get dict=q[at.parent.type]
        #   find the atom's name in dict to get a charge
        #   set atom._charges['Kollman'] to that charge
        #   there are special problems with his, cys and termini
        #   set allAtoms.chargeSet to 'Kollman'
        if needToAdd:     
            if not hasattr(self, 'isPeptide'):
                msg = self.checkIsPeptide()
            if self.isPeptide:
                #self.checked_charges = 'needs Kollman'
                self.addKollman()
                self.checked_charges = 'has Kollman'
                #self.vf.addKollmanCharges(self, topCommand=0, log=0)
                msg = msg + '    -added Kollman charges\n'
            else:
                #self.checked_charges = 'needs gasteiger'
                self.computeGasteiger()
                self.checked_charges = 'has gasteiger'
                #self.vf.computeGasteiger(self, topCommand=0, log=0)
                msg = msg + '    -added gasteiger charges\n'
        else:
            self.checked_charges = 'has charges'
            msg = msg + '   -already had charges\n'
        # adt will have to add charges... AND check them
        #at this point, every atom has a charge and charges are not all 0k
        #have to have unit charges per residue
        #print 'calling checkMolCharges'
        #errCharge, resList = checkMolCharges(self, self.vf)
        #FIX THIS: need to add mechanism to adjust charges
        #gui to let user pick + change
        #or either change on first atom or spread over all
        #self.checked_charges = 1
        #if added charges, msg will say which one
        #if did't add charges, msg will be 'ERROR'
        return msg


    def computeGasteiger(self):
        #to compute Gasteiger need to:
        #   create an AtomHybridization()
        #   call its assignHybridization method on self.allAtoms
        #   create a Gasteiger()
        #   call its compute method on self.allAtoms
        # THEN move gast_charge into _charges with gasteiger key
        #   set allAtoms.chargeSet to 'gasteiger'
        # THEN delattr gast_charge from allAtoms
        allAts = self.allAtoms
        ah = AtomHybridization()
        ah.assignHybridization(allAts)
        Gast = Gasteiger()
        Gast.compute(allAts)
        gastCharges = []
        for c in allAts.gast_charge:
            gastCharges.append(round(c, 3))
        allAts.addCharges('gasteiger', gastCharges)
        del allAts.gast_charge
        allAts.chargeSet = 'gasteiger'


    def addKollman(self):
        #to add Kollman need to:
        #   look up each atom's parent in q to get dict=q[at.parent.type]
        #   find the atom's name in dict to get a charge
        #   set atom._charges['Kollman'] to that charge
        #   there are special problems with his, cys and termini
        #   set allAtoms.chargeSet to 'Kollman'

        for a in self.allAtoms:
            dict = q.get(a.parent.type, {})
            if len(dict):
                a._charges['Kollman'] = dict.get(a.parent.type, 0)
            else:
                a._charges['Kollman'] = 0
            a.chargeSet = 'Kollman'


    def checkHydrogens(self):
        """
        checkHydrogens
        """
        #what if self doesn't have bonds
        if not self.chains[0].hasBonds:
            self.buildBondsByDistance()
        hs = self.allAtoms.get(lambda x: x.element=='H')
        self.nphs = AtomSet()
        if not hs:
            msg = '    -no polar hydrogens found!\n'
        if hs:
            nphs = hs.get(lambda x: x.bonds[0].atom1.element=='C' or \
                    x.bonds[0].atom2.element=='C')
            if nphs:
                self.nphs = nphs
                msg = '    -found ' + str(len(self.nphs)) + ' nonpolar hydrogens\n'
            else:
                msg = '    -no nonpolar hydrogens\n'
        return msg


    def mergeNPHS(self):
        lenNPHS = len(self.nphs)
        if not lenNPHS:
            return
        nphs = self.nphs
        atLen = len(self.allAtoms)
        self.allAtoms = self.allAtoms - nphs
        #first add charge to nph's heavy atom
        chList = nphs[0]._charges.keys()
        if not len(chList):
            print 'no charges present'
            return 'XXX'
        #check that all nphs have a certain kind of charge
        for at in nphs:
            chs = at._charges.keys()
            for c in chList:
                if c not in chs:
                    chList.remove(c)
        if len(chList):
            for chargeSet in chList:
                for h in nphs:
                    b = h.bonds[0]
                    c = b.atom1
                    if c==h: 
                        c = b.atom2
                    c._charges[chargeSet] = c._charges[chargeSet] + h._charges[chargeSet]
        #next delete nphs
        for h in nphs:
            b = h.bonds[0]
            c = b.atom1
            if c==h: 
                c = b.atom2
            c.bonds.remove(b)
            h.bonds = BondSet()
            h.parent.remove(h)
            #nb: 
            #these don't show up in deleteMol.getFreeMemoryInformation
            del h
        self.nphs = AtomSet()
        return '        and merged them\n', nphs


    def checkBonds(self):
        """
        checkBonds
        """
        msg = ""
        #print self.name, ':'
        nphs = AtomSet()
        if len(self.nphs) and self.autoMergeNPHS:
            newmsg, nphs = self.mergeNPHS()
            msg = msg + newmsg
        bc = self.bondClassifier
        bonds = self.chains.residues.atoms.bonds[0]
        for b in bonds:
            b.activeTors = 0
            b.possibleTors = 0
            b.leaf = 0
        #FIX THIS:
        #if isPeptide, don't get cycles this way
        #if self.isPeptide:
            #cycleSelector = bc['cycles']
            #del bc['cycles']
        results = bc.classify(bonds)
        #if self.isPeptide:
            #bc['cycles'] = cycleSelector
            #results['cycles'] = self.getPeptideBondDict()
        #for k,v in results.items():
        #    print k,' = ', len(v)
        self.rotatable = results['rotatable']
        self.torscount = len(self.rotatable) 
        self.possible_torscount = self.torscount
        for b in self.rotatable:
            b.activeTors = 1
            b.possibleTors = 1
        for b in results['leaf']:
            b.leaf = 1
        for b in results['cycle']:
            b.incycle = 1
        hydrogenRotators = results['hydrogenRotators']
        self.hydrogenRotators = hydrogenRotators 
        self.TORSDOF = self.torscount - len(hydrogenRotators)
        ptAts = bc.dict['rotatable'].getAtoms(self.rotatable)
        d = {}
        for a in ptAts:
            d[a] = 0
        self.pTatomset = AtomSet(d.keys())
        #print 'len(pTatomset=', len(self.pTatomset)
        self.leafbonds = results['leaf']
        self.pepbackbonds = results['ppbb']
        self.amidebonds = results['amide']
        self.cyclebonds = results['cycle']
        return msg, nphs


    def checkAromatics(self):
        """
        checkAromatics
        """
        #this depends on userPref useProteinAromaticList
        if not len(self.cyclebonds):
            self.aromaticCs = AtomSet()
            return ""
        if self.isPeptide and self.useProteinAromaticList:
            self.aromaticCs = self.getPeptideAromatics()
            return ""
        if self.isPeptide:
            self.getPeptideBondDict()
        else:
            self.getLigandBondDict()
        counter = 0
        while counter < self.cyclecount:
            counter = counter + 1
            blist = self.bondDict[counter]
            for item in blist:
                at = item.atom1
                self._getAdjAtom(item, blist)
                #now each bond has 3 atoms specified for it: its own two and the next1 to atom1
                result = self._getNormal(item)
                item.nrmsize = result[0]
                item.nrms = result[1]
                #next find the other bond w/atom2:
                z2 = filter(lambda x,item=item, at2=item.atom2, blist=blist:x!=item and x.atom1==at2 or x.atom2==at2, blist)
                #finally, return the other atom in this bond
                item.nextbond2 = z2[0]
                if item.nextbond2==item:
                    item.nextbond2 = z2[1]
                neighbor2 = self._getnxtAtom(item.atom2,item.nextbond2)
                item.next2 = neighbor2
                #next have to check whether the normals are parallel
                #check each pair in each bond, how to keep track??
                #have to get normal at item's atom2: so have to get next2 for this bond:
            #have to loop twice to make sure neighbor has nrms
            for item in blist:
                p = item.nrms
                psize = item.nrmsize
                q = item.nextbond2.nrms
                qsize = item.nextbond2.nrmsize
                #theta is the critical test for planarity:
                #if angle between 2 nrms is 0, atoms are planar
                #NB>test is comparing theta,cos(angle), w/zero
                item.theta = Numeric.dot(p,q)/(psize*qsize)
                for p in ['next1','next2','nextbond','nextbond2']:
                    delattr(item, p)
        self.updateAromatics(self.aromaticCutOff)
        msg = '    -found '+ str(len(self.aromaticCs)) + ' aromatic carbons\n'
        return  msg


    def updateAromatics(self, cutoff):
        self.aromaticCutOff = cutoff
        #cutoff = self.aromaticCutOff
        cutoffValue = math.cos(cutoff*math.pi/180.)
        aromaticCs = AtomSet([])
        atD = {}
        #to keep from overwriting names of aromatic carbons at 
        #junctions of rings use this klug
        for blist in self.bondDict.values():
            for item in blist:
                item.atom1.setThisTime = 0
                item.atom2.setThisTime = 0
        for blist in self.bondDict.values():
            ct = 0
            for item in blist:
                #these are values for the default 7.5degrees:
                #if theta>=0.997 or theta<=-0.997:
                if item.theta>=cutoffValue or item.theta<=-cutoffValue:
                    item.posAromatic = 1
                    ct = ct + 1
                else:
                    item.posAromatic = 0
            #after checking all the bonds in current cycle, compare #posAromatic w/number
            if ct==len(blist):
                #and change the name and autodock_element here....
                for b in blist:
                    at1 = b.atom1
                    at2 = b.atom2
                    if at1.element=='C':
                        at1.name = 'A' + at1.name[1:]
                        at1.autodock_element = 'A'
                        atD[at1] = 0
                        at1.setThisTime = 1
                    if at2.element=='C':
                        at2.name = 'A' + at2.name[1:]
                        at2.autodock_element = 'A'
                        atD[at2] = 0
                        at2.setThisTime = 1
                aromaticCs = AtomSet(atD.keys())
            else:
                #if there were any aromatic carbons which no longer 
                #meet the criteria, change them back
                for b in blist:
                    at1 = b.atom1
                    at2 = b.atom2
                    if at1.name[0]=='A' and not at1.setThisTime:
                        at2.autodock_element = 'C'
                        at1.name = 'C' + at1.name[1:]
                    if at2.name[0]=='A'and not at2.setThisTime:
                        at2.autodock_element = 'C'
                        at2.name = 'C' + at2.name[1:]
        #remove klug
        for blist in self.bondDict.values():
            for item in blist:
                if hasattr(item.atom1, 'setThisTime'):
                    delattr(item.atom1,'setThisTime')
                if hasattr(item.atom2, 'setThisTime'):
                    delattr(item.atom2,'setThisTime')
        for a in aromaticCs:
            a.autodock_element = 'A'
        self.aromaticCs = aromaticCs

            
    def restoreCarbons(self):
        for a in self.aromaticCs:
            if len(a.name)==1:
                a.name = 'C'
            else:
                a.name = 'C' + a.name[1:]
        a.autodock_element = 'C'

   
    def nameAromatics(self):
        for a in self.aromaticCs:
            if len(a.name)==1:
                a.name = 'A'
            else:
                a.name = 'A' + a.name[1:]
        a.autodock_element = 'A'


    def addAromatic(self, at):
        if at.element!='C':
            print at.name, ' is not a carbon'
            return 'ERROR'
        if at in self.aromaticCs:
            print at.name, ' is already in aromatic set'
            return 'ERROR'
        at.autodock_element = 'A'
        self.aromaticCs.append(at)
        print at.name, ' added to aromatic set'
   

    def removeAromatic(self, at):
        if at not in self.aromaticCs:
            print at.name, ' is not in aromatic set'
            return 'ERROR'
        self.aromaticCs.remove(at)
        at.autodock_element = 'C'
        print at.name, ' removed from aromatic set'
   

    def getPeptideAromatics(self):
        atSet = AtomSet()
        allAts = self.allAtoms
        arom_ats = allAts.get(lambda x, l = pep_aromList:\
            x.parent.type+'_'+x.name in l)
        if not arom_ats:
            return AtomSet([])
        for at in arom_ats:
            at.name = 'A' + at.name[1:]
            at.autodock_element = 'A'
        #print 'returning ', len(arom_ats), ' arom_ats'
        return arom_ats

       
    def getPeptideBondDict(self):
        resSet = self.chains.residues.get(lambda x, \
                    d = aromDict.keys():x.type in d)
        if not resSet:
            return BondSet()
        self.cyclecount = numres = len(resSet)
        #NB: cyclenum is 1 based because 0 means not numbered
        for i in range(numres):
            res = resSet[i]
            ats = res.atoms
            #bondDict keys are 1 based too
            keys = aromDict[res.type]
            bnds = bondDict[i+1] = ats.get(lambda x, \
                                keys=keys:x.name in keys).bonds[0]
            for b in bnds:
                bnds.cyclenum = i + 1
        self.bondDict = bondDict


    def getLigandBondDict(self):
        ats = self.allAtoms
        babel = AtomHybridization()
        babel.assignHybridization(self.allAtoms)
        #typeBonds???
        #typeAtoms(ats)
        rf = RingFinder()
        rf.findRings2(ats, ats.bonds[0])
        ct = 1
        bondDict = {}
        for ring in rf.rings:
            bondDict[ct] = ring['bonds']
            for b in ring['bonds']:
                b.cyclenum = ct
            ct = ct + 1
        self.cyclecount = rf.ringCount
        self.bondDict = bondDict

 
    def _numberCycle(self, at):
        for b in at.bonds:
            if hasattr(b,'incycle') and b.cyclenum==0 :
                b.cyclenum = self.cyclecount
                nxtAtom = b.atom1
                if nxtAtom==at:
                    nxtAtom = b.atom2
                self._numberCycle(nxtAtom)


    def _getnxtAtom(self, at,b):
        nxtAtom = b.atom1
        if nxtAtom==at:
            nxtAtom = b.atom2
        return nxtAtom


    def _getAdjAtom(self,b,z):
        #first get the bonds in this cycle
        at = b.atom1
        #next find the other one with at as one of the atoms:
        z2 = filter(lambda x,b=b,at=at,z=z:x!=b and x.atom1==at or x.atom2==at, z)
        #finally, return the other atom in this bond
        b.nextbond = z2[0]
        neighbor = self._getnxtAtom(at,z2[0])
        b.next1 = neighbor


    def _getNormal(self,b):
        at1 = b.next1
        at2 = b.atom1
        at3 = b.atom2
        pt1 = at1.coords
        pt2 = at2.coords
        pt3 = at3.coords
        a = Numeric.subtract(pt2,pt1)
        b = Numeric.subtract(pt3,pt2)
        p = (a[1]*b[2]-a[2]*b[1],a[2]*b[0]-a[0]*b[2],a[0]*b[1]-a[1]*b[0])
        nrmsize = Numeric.sqrt(p[0]*p[0]+p[1]*p[1]+p[2]*p[2])
        return (nrmsize, p)


    def setroot(self, atom):
        """
        setroot to 'C11' or 'hsg1:A:ILE2:CA'
        """
        if type(atom)==types.StringType:
            if find(atom, ':')>-1:
                #have to use full_name list
                #check that it is an atom
                mols = ProteinSet([self])
                nodes = mols.NodesFromName(atom)
                if not nodes:
                    return 'invalid root name'
                if nodes.__class__!=AtomSet:
                    return 'invalid root name: not atom level'
            else:
                nodes = self.allAtoms.get(lambda x, n = atom:x.name==n)
            if not nodes:
                return 'invalid root name'
            if len(nodes)>1:
                return 'root name must be unique'
            atom = nodes[0]
        #elif type(atom)!=types.InstanceType:
        elif isInstance(atom) is False:
            return atom, ' invalid root atom'
        elif atom.__class__!=Atom:
            return atom, ' can only select an Atom instance as root'
        #fix this rnum0 stuff
        #in case toggling back and forth
        if hasattr(self, 'autoRoot') and hasattr(self.autoRoot, 'rnum0'):
            delattr(self.autoRoot, 'rnum0')
        #if there is an old root, remove rnum0
        if hasattr(self, 'ROOT') and hasattr(self.ROOT, 'rnum0'):
            delattr(self.ROOT, 'rnum0')
        self.ROOT = atom
        self.ROOT.rnum0 = 0
        return self.ROOT


    def autoroot(self):
        """
        autoRoot
        """
        #clear old root
        if hasattr(self, 'ROOT') and hasattr(self.ROOT, 'rnum0'):
            delattr(self.ROOT, 'rnum0')
        if hasattr(self, 'autoRoot'):
            self.ROOT = self.autoRoot
            self.ROOT.rnum0 = 0
            return
        if len(self.chains)>1:
            return  "AutoRoot not implemented for molecules with >1 chain"
        self.bestBranch = len(self.allAtoms)
        bestList = []
        for item in self.allAtoms:
            if not hasattr(item, 'bonds'): continue
            if len(item.bonds)==1 and item.bonds[0].leaf: continue
            if hasattr(item,'leaf'): continue
            item.maxbranch = 0
            for b in item.bonds:
                nxtAtom = self._getnxtAtom(item,b)
                if not b.leaf:
                    thistree = self.subTree(item,nxtAtom, self.allAtoms)
                    thisbranch = len(thistree)
                    if thisbranch>item.maxbranch:
                        item.maxbranch = thisbranch
            #bestList holds all current best choices for Root..
            if item.maxbranch<self.bestBranch:
                bestList = []
                bestList.append(item)
                self.bestBranch = item.maxbranch
            if item.maxbranch==self.bestBranch and item not in bestList:
                bestList.append(item)
        if len(bestList)>1:
            foundCycle = 0
            for at in bestList:
                at.cycleatom = 0
                for b in at.bonds:
                    #4/29: peptides won't have aromatic 
                    #but will have incycle
                    #if b.bondOrder=='aromatic':
                    if hasattr(b,'incycle'):
                        at.cycleatom = 1
                        continue
            for at in bestList:
                if at.cycleatom:
                    self.ROOT = at
                    self.autoRoot = at
                    self.ROOT.rnum0 =0
                    foundCycle = 1
                    break
            #if bestList had a cycle atom, it's been set to root..if NOT:
            if not foundCycle:
                self.autoRoot = bestList[0]
                self.ROOT = bestList[0]
                self.ROOT.rnum0 =0
        #no ties for possible root, use bestRoot...
        else:
            self.autoRoot = bestList[0]
            self.ROOT = bestList[0]
            self.ROOT.rnum0 =0
        return self.ROOT


    def buildTorsionTree(self):
        if not hasattr(self, 'ROOT'):
            print 'must set ROOT first!'
            return
        self.torTree = TorTree(self.parser, self.ROOT)


    def turnOnTorsion(self, bnd):
        """
        turnOnTorsion 
        """
        if not bnd in self.rotatable:
            return
        #this is redundant (??)
        if not bnd.possibleTors:
            return
        if not bnd.activeTors:
            self.torscount = self.torscount + 1
            bnd.activeTors = 1
            self.torTree.addTorsion(bnd.atom1.number, bnd.atom2.number)


    def turnOffTorsion(self, bnd):
        """
        turnOffTorsion 
        """
        if not bnd in self.rotatable:
            return
        #this is redundant (??)
        if not bnd.possibleTors:
            return
        if bnd.activeTors:
            self.torscount = self.torscount - 1
            bnd.activeTors = 0
            self.torTree.removeTorsion(bnd.atom1.number, bnd.atom2.number)


    def limitTorsions(self, numTors, type, simpleModel=1):
        """sets number of torsions to specified number by inactivating either those which
move the fewest atoms or those which move the most. if number is > than
current but less than possible, torsions are reactivated
        """

        print 'lT: numTors=', numTors, ' type=', type
        allAts = self.allAtoms
        root = self.ROOT
        torscount = self.torscount
        print 'torscount=', torscount, 
        #NB: torscount is not necessarily the max
        #ie it could have been adjusted already
        at0 = self.allAtoms[0]
        if not hasattr(self, 'torTree') or not hasattr(at0, 'tt_ind'):
            self.torTree = TorTree(self.parser, root)
        torsionMap = self.torTree.torsionMap
        torsionMapNum = len(torsionMap)
        print 'len(tm)=', torsionMapNum
        possibleTors = self.possible_torscount
        if simpleModel:
            self.setTorsions(numTors, type)
            return 

        #FIX THIS: what if want to increase active torsions
        if torscount==numTors:
            msg = 'specified number==number present: no adjustment'
            return 'msg'
        elif torscount<numTors:
            if torscount==possibleTors:
                #if torscount==torsionMapNum:
                msg = 'specified number == number possible: no adjustment'
                return msg
            else:
                #in this case turn on as many as possible
                #if numTors>=torsionMapNum:
                if numTors>=possibleTors:
                    #turn on everything
                    delta = possibleTors - torscount
                    #delta = torsionMapNum - torscount
                else:
                    delta = numTors - torscount
                self.turnOnTorsions(delta, type)
        else:
            #torscount>numTors
            #in this case turn them off 
            delta = torscount - numTors
            self.turnOffTorsions(delta, type)
        msg = str(delta) + ' torsions changed'
        return msg


    def setTorsions(self, numTors, type):
        #this assumes atoms have tt_ind field
        tNum = len(self.torTree.base_torsionMap)
        msg = ""
        if numTors>tNum:
            msg = 'too many torsions specified! '+ str(numTors)+  ' reducing to'+str(tNum)
            numTors = tNum
        if type=='fewest':
            rangeList = range(numTors)
        else:
            rangeList = []
            for k in range(1, numTors+1):
                rangeList.append(-k)
        #turn them all off
        baseTorsionMap = self.torTree.base_torsionMap
        torsionMap = self.torTree.torsionMap
        #turn all active nodes off
        active_nodes = torsionMap[:]
        for node in active_nodes:
            #print 'turning off node ', node.number
            self.torTree.removeTorsion(node.bond[0], node.bond[1])
            b = self.allAtoms.get(lambda x, node=node: x.tt_ind in node.bond).bonds[0][0]
            b.activeTors = 0

        #turn on the right number at correct end
        for i in rangeList:
            node = baseTorsionMap[i]
            #print '2:turning on node ', node.number
            self.torTree.addTorsion(node.bond[0], node.bond[1])
            b = self.allAtoms.get(lambda x, node=node: x.tt_ind in node.bond).bonds[0][0]
            b.activeTors = 1
            #print 'now len(torsionMap)=', len(torsionMap)
        self.torscount = numTors
        return msg


    def turnOnTorsions(self, delta, type = 'fewest'):
        allAts = self.allAtoms
        torsionMap = self.torTree.torsionMap
        baseTorsionMap = self.torTree.base_torsionMap
        torscount = self.torscount
        #turn on delta torsions + adjust torscount in dict 
        if type=='fewest':
            rangeList = range(delta)
        else:
            rangeList = []
            for k in range(1, delta+1):
                rangeList.append(-k)
        #FIX THIS: probably doesn't work
        for i in rangeList:
            node = baseTorsionMap[i]
            b = allAts.get(lambda x, node=node: x.tt_ind in node.bond).bonds[0][0]
            if not b.activeTors:
                self.torTree.addTorsion(node.bond[0], node.bond[1])
                b.activeTors = 1
            else:
                lastInd = rangeList[-1]
                if type=='fewest':
                    rangeList.append(lastInd+1)
                else:
                    rangeList.append(lastInd-1)
        #torscount should be torscount + delta here
        self.torscount = torscount + delta

        
    def turnOffTorsions(self, delta, type = 'fewest'):
        allAts = self.allAtoms
        torsionMap = self.torTree.torsionMap
        baseTorsionMap = self.torTree.base_torsionMap
        torscount = self.torscount
        #turn on delta torsions + adjust torscount in dict 
        if type=='fewest':
            rangeList = range(delta)
        else:
            rangeList = []
            for k in range(1, delta+1):
                rangeList.append(-k)
        for i in rangeList:
            node = baseTorsionMap[i]
            if node.bond==(None,None):
                print 'error in turnOff torsions with ', rangeList
                break
            b = allAts.get(lambda x, node=node: x.tt_ind in node.bond).bonds[0][0]
            if b.activeTors:
                self.torTree.removeTorsion(node.bond[0], node.bond[1])
                b.activeTors = 0
            else:
                lastInd = rangeList[-1]
                if type=='fewest':
                    rangeList.append(lastInd+1)
                else:
                    rangeList.append(lastInd-1)
        self.torscount = torscount - delta


    def setCarbonName(self, at):
        """
        setCarbonName 
        """
        if not at.element=='C':
            return
        if len(at.name)==1:
            at.name = 'C'
        else:
            at.name = 'C' + at.name[1:]



    def setAromaticName(self, at):
        """
        setAromaticName 
        """
        if not at.element=='C':
            return
        if len(at.name)==1:
            at.name = 'A'
        else:
            at.name = 'A' + at.name[1:]





# The .packager.py is used by the DistTools package to get the files to be
# included in a distribution.

def getFiles(root, what = 'all', plat=None):
    """
    files <- getFiles(root, what='all', plat=None)

    files -- list of the files to be included in the download

    arguments:
    root -- path to the package. This is used by glob to get all the python
            modules.
    what -- string that can be 'all' and 'supported' to specified what files
    to include in the distribution. By default 'all' the files are added.
    plat -- platform ('linux2', 'irix646', 'sunos5' etc...)
    """
    import os
    from glob import glob 

    # 1- Specify the list of all the Python module
    allPyModule = ["*.py"]

    # 2- Specify the list of the non supported Python module. These files
    # will be removed from the release of the supported python modules.
    pynotsupported = []

    # 3- Specify the documentation files and directories to be included in the
    # release
    docFiles = ["Tutorial"]
    
    # 4-Specify the extraFiles to be included in the release.
    extraFiles = ["CVS/",
                  'bin/CVS',
                  'bin/runAdt.py']

    # 5-Specify the testFiles to be included in the release.
    testFiles = ["Tests"]


    #########################################################
    ## Where things are done for you .
    #########################################################
    # if some files need to be removed, we need the exact list of the pymodule.
    if len(pynotsupported):
        # store the path of the current directory
        olddir = os.getcwd()
        os.chdir(root)
        files = []
        # we use glob to get the exact list of files.
        for p in allPyModule:
            files = files + glob(p)
        allPyModule = files
        files = []
        # need to get the list of the files ... no wild card possible.
        for p in pynotsupported:
            files = files + glob(p)
        pynotsupported = files
        os.chdir(olddir)
    # Creation of the proper list of files depending on the value of what
    
    if what == 'supported' and len(pynotsupported):
        # need to remove the non supported python files from all the python
        # files
        # These are the keys used for to make the releases...
        supportedFiles = filter(lambda x, l = pynotsupported:
                                    not x in l, allPyModule)
        
        return supportedFiles +  testFiles + extraFiles
    
    elif what == 'all' or ( what == 'supported' and not len(pynotsupported)):
        # Other wise just add the documentation, test and extra files to all
        # the python modules.
        allFiles= allPyModule + docFiles + testFiles + extraFiles
        return allFiles
    elif what == 'documentation':
        return docFiles
    

    else:
        return []




#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/summarize_time.py,v 1.4 2012/02/28 18:17:01 rhuey Exp $
#
import os, glob, numpy.oldnumeric as Numeric
from AutoDockTools.DlgParser import DlgParser


if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: summarize_time.py -d directory"
        print
        print "    Description of command..."
        print "         -d     directory"
        print "    Optional parameters:"
        print "        [-o]    output filename"
        print "                      (default is 'summary_of_time')"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'd:o:vh')
    except getopt.GetoptError, msg:
        print 'summarize_time.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-d: directory
    directory =  None

    # optional parameters
    #-o outputfilename
    outputfilename = "summary_of_time.txt"
    #-v: verbose
    verbose = None
    #-h: help
    help = None


    #'d:o:vh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-d', '--d'):
            directory = a
            if verbose: print 'set directory to ', a
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    if not  directory:
        print 'summarize_time: directory must be specified.'
        usage()
        sys.exit()

    p = DlgParser()
    dlg_list = glob.glob(directory + '/*.dlg')
    total_time = 0
    for dlg in dlg_list:
        p.parse(dlg)
        total_time = total_time + p.total_time
        if verbose: print "dlg time=", total_time
    stem = dlg_list[0].split('.')[0]
    mins = total_time/60.
    if verbose: print "total time in minutes =", mins
    ostr = '%s: % 12.4f m \n' %(directory, mins)
    fptr = open(outputfilename, 'a')
    fptr.write(ostr)
    fptr.close()

# To execute this command type:
# summarize_time.py -d directory -o outputfilename -v




#########################################################################
#
# Date: Nov 2001 Authors: Michel Sanner
#
#    sanner@scripps.edu
#
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Michel Sanner and TSRI
#
#########################################################################

from NetworkEditor.items import NetworkNode
from AutoDockTools.VisionInterface.Adt.dpf_template import dpf_template

class MakeDPFCopies(NetworkNode):
    """
    A node that takes a DPF and a directory of pdb/pqr/pdbqt files and 
    makes copies of the DPF.  The DPF copies will have the basenames of the
    pdb/pqr/pdbqt files.  
  
    Input 1: DPF template file with the extension .dpf
    Input 2: A directory path to the PDB/PQR/PDBQT files
    Output:  List of all DPFs in the directory
    """
    
    def __init__(self, name='MakeDPFCopies', **kw):
        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )

        kw['name'] = name
        apply( NetworkNode.__init__, (self,), kw )
        ip = self.inputPortsDescr
        ip.append(datatype='dpf_template', name='dpf_file')
        ip.append(datatype='string', name='struct_dir')

        op = self.outputPortsDescr
        op.append(datatype='list', name='dpf_list')

        code = """def doit(self, dpf_file, struct_dir):
    import os
    import shutil
    from AutoDockTools.VisionInterface.Adt.dpf_template import dpf_template

    file_path = dpf_file.fullpath

    if not(os.path.exists(file_path)):
        print "ERROR: DPF template file " + file_path + " does not exist!"
        return 'stop'

    name_list = set()
    dpf_list = []

    d = os.path.abspath(struct_dir)

    for i in os.listdir(struct_dir):
        if i.endswith(".pdbqt") or i.endswith(".pdb") or i.endswith(".pqr"):
            name_list.add(i.split('.')[0])

    for i in name_list:
        dst = os.path.join(d, i + '.dpf')
        dpf_list.append(dst)       

        if not(os.path.exists(dst)):
            shutil.copyfile(file_path, dst)
  

    self.outputData(dpf_list=dpf_list)
"""
        self.setFunction(code)



#
#
#
#
# $Id: test_pdbq_to_pdbqt.py,v 1.1 2005/07/21 22:47:02 sowjanya Exp $
#

import unittest, os, sys
from string import split, find
from MolKit import Read

class BaseTests(unittest.TestCase):

    def test_default_formatted_pdbq(self):
        """
    pdbq_to_pdbqt.py -s pdbq_stem -o outputfilename
       -s ligand_stem
       optional:
       [-o outputfilename]

        """
        filename = "test_formatted.pdbqt"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd = cmd + '; ' + "../pdbq_to_pdbqt.py -s 1ebg_lig -o "+ filename
        #cmd = "../pdbq_to_pdbqt.py -s 1ebg_lig -o "+ filename
        os.system(cmd)
        self.assertEqual(os.path.exists(filename), True)
        fptr = open('ref_formatted.pdbqt')
        reflines = fptr.readlines()
        fptr.close()
        fptr = open(filename)
        testlines = fptr.readlines()
        fptr = open(filename)
        for refline, testline in zip(reflines, testlines):
            if find(refline, 'REMARK')<0:
                self.assertEqual(refline, testline)


    def test_default_plain_pdbq(self):
        """
    pdbq_to_pdbqt.py -s pdbq_stem -o outputfilename
       -s ligand_stem
        """
        filename = "test_plain.pdbqt"
        if os.path.exists(filename):
            cmd = "rm  " + filename
            os.system(cmd)
        cmd = 'set PYTHONPATH=%s'%sys.path[0]
        cmd = cmd + '; ' + "../pdbq_to_pdbqt.py -s 1ebg -o "+ filename
        #cmd = "../pdbq_to_pdbqt.py -s 1ebg  -o " + filename
        os.system(cmd)
        self.assertEqual(os.path.exists(filename), True)
        fptr = open('ref_plain.pdbqt')
        reflines = fptr.readlines()
        fptr.close()
        fptr = open(filename)
        testlines = fptr.readlines()
        fptr = open(filename)
        for refline, testline in zip(reflines, testlines):
            self.assertEqual(refline, testline)



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/compute_AutoDock41_score.py,v 1.12 2012/04/05 21:00:20 rhuey Exp $
#
import os 

from MolKit import Read
import MolKit.molecule
import MolKit.protein


from PyAutoDock.InternalEnergy import InternalEnergy
from PyAutoDock.MolecularSystem import MolecularSystem
from PyAutoDock.AutoDockScorer import AutoDock41Scorer

def check_types(molecule,std_types):
    d = {}
    for a in molecule.allAtoms:
        d[a.autodock_element] = 1
    mol_types = d.keys()
    non_std = []
    for t in mol_types:
        if t not in std_types:
            non_std.append(t)
    return non_std            


if __name__ == '__main__':
    import sys
    import getopt


# compute_AutoDock41_score.py -r receptorfilename -l ligandfilename [-o outputfilename -x exclude_torsFreeEnergy -w write_file] -v
    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: compute_AutoDock41_score.py -r receptorfilename -l ligandfilename"
        print
        print "    Description of command..."
        print "      Calculates the AutoDock41 energy between two molecules and stores it in 'AutoDock41_scores.txt'"
        print "      By default uses 'AD4.1_bound.dat' parameters and includes torsFreeEnergy."
        print "         -r     receptorfilename"
        print "         -l     ligandfilename"
        print "    Optional parameters:"
        print "        [-o]    outputfilename (default is 'AutoDock41_scores.txt')"
        print "        [-p]    parameter_library_file (default built-in values from 'AD4.1_bound.dat')"
        print "        [-x]    exclude_torsFreeEnergy (default is to include the loss of torsional entropy)"
        print "        [-w]    write_file_mode (default mode is 'a')"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'r:l:wvxo:p:h')
    except getopt.GetoptError, msg:
        print 'compute_AutoDock41_score.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-r: receptorfilename
    receptorfilename =  None
    #-l: ligandfilename
    ligandfilename =  None
    #-w write new outputfile
    write_file_mode = False
    # optional parameters
    #-o outputfilename
    outputfilename = 'AutoDock41_scores.txt'
    #-p parameter_library_filename
    parameter_library_filename = None
    #-x exclude_torsFreeEnergy, default is to include torsFreeEnergy
    exclude_torsFreeEnergy = False
    #-v verbose
    verbose = None

    #'r:l:wvxo:p:h'
    for o, a in opt_list:
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-r', '--r'):
            receptorfilename = a
            if verbose: print 'set receptorfilename to ', a
        if o in ('-l', '--l'):
            ligandfilename = a
            if verbose: print 'set ligandfilename to ', a
        if o in ('-w', '--w'):
            write_file_mode = True
            if verbose: print 'set write_file_mode to ', True
        if o in ('-x', '--x'):
            exclude_torsFreeEnergy = True
            if verbose: print 'set exclude_torsFreeEnergy to ', exclude_torsFreeEnergy
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-p', '--p'):
            parameter_library_filename = a
            if verbose: print 'set parameter_library_filename to ', a
        if o in ('-h', '--'):
            usage()
            sys.exit()


    if not receptorfilename:
        print 'compute_AutoDock41_score: receptorfilename must be specified.'
        usage()
        sys.exit()

    if not ligandfilename:
        print 'compute_AutoDock41_score: ligandfilename must be specified.'
        usage()
        sys.exit()

    ad_scorer = AutoDock41Scorer(exclude_torsFreeEnergy=exclude_torsFreeEnergy)
    if parameter_library_filename is not None:
        ad_scorer.read_parameter_file(parameter_library_filename)
    supported_types = ad_scorer.supported_types
    receptor = Read(receptorfilename)[0]
    assert os.path.splitext(receptor.parser.filename)[-1]=='.pdbqt',"receptor file not in required '.pdbqt' format"
    if verbose: print 'read ', receptorfilename
    receptor.buildBondsByDistance()
    rec_non_std = ""
    non_std_types = check_types(receptor, supported_types)
    if len(non_std_types):
        rec_non_std = non_std_types[0]
        if len(non_std_types)>1:
            for t in non_std_types[1:]:
                rec_non_std = rec_non_std + '_' + t
    #check format of receptor
    ligand = Read(ligandfilename)[0]
    assert os.path.splitext(ligand.parser.filename)[-1]=='.pdbqt',"ligand file not in required '.pdbqt' format"
    if verbose: print 'read ', ligandfilename
    ligand.buildBondsByDistance()
    lig_non_std = ""
    non_std_types = check_types(ligand, supported_types)
    if len(non_std_types):
        lig_non_std = non_std_types[0]
        if len(non_std_types)>1:
            for t in non_std_types[1:]:
                lig_non_std = lig_non_std + '_' + t
    mode = 'a'
    first = not os.path.exists(outputfilename)
    if write_file_mode:
        mode = 'w'
        first = True
    if verbose: print 'first is ', first
    optr = open(outputfilename, mode)
    if first:
        tstr = "    Receptor      Ligand   AutoDock4.1Score     estat      hb      vdw   dsolv    tors\n"
        optr.write(tstr)
    #setup the molecular system
    ostr = ""
    if len(lig_non_std):
        ostr = 'ERROR: unable to score ligand "%s" due to presence of non-standard atom type(s): %s\n' %(ligand.name, lig_non_std)
        optr.write(ostr)
    elif len(rec_non_std):
        ostr = 'ERROR: unable to score receptor "%s" due to non-standard atom type(s): %s\n' %(receptor.name, rec_non_std)
        optr.write(ostr)
    else: 
        ms = MolecularSystem()
        ms.add_entities(receptor.allAtoms)
        ms.add_entities(ligand.allAtoms)
        ad_scorer.set_molecular_system(ms)
        #get the scores
        #score per term
        estat, hb,vdw,dsolv = ad_scorer.get_score_per_term()
        torsEnrg = ligand.TORSDOF * ad_scorer.tors_weight
        score = estat + hb + vdw + dsolv + torsEnrg
        ostr = "%12s%12s      % 8.4f        % 6.4f % 6.4f % 6.4f % 6.4f % 6.4f\n" %(receptor.name,ligand.name, score, estat, hb, vdw, dsolv, torsEnrg)
        optr.write(ostr)
        if verbose:
            print 'wrote %s' %outputfilename
    optr.close()

# To execute this command type:
# compute_AutoDock41_score.py -r receptorfilename -l ligandfilename [-o outputfilename -x exclude_torsFreeEnergy -w write_file_mode] -v




